SystemOrganization addCategory: #'Magma tester-Benchmarks'!
SystemOrganization addCategory: #'Magma tester-Info'!
SystemOrganization addCategory: #'Magma tester-Notifications'!
SystemOrganization addCategory: #'Magma tester-SUnit tests'!
SystemOrganization addCategory: #'Magma tester-Test domains'!

!MaKeywordIndex methodsFor: '*magma tester' stamp: 'cmm 7/2/2006 22:38'!
ascendingSortBlock
	^ [ : a : b | (self indexHashesFor: a) min < (self indexHashesFor: b) min ]! !

!PositionableStream methodsFor: '*magma tester' stamp: 'cmm 9/17/2006 21:08'!
maFileInAnnouncing: announcement logged: aBoolean
	"This is special for reading expressions from text that has been formatted 
	with exclamation delimitors. The expressions are read and passed to the 
	Compiler. Answer the result of compilation.  Put up a progress report with
     the given announcement as the title."
	| val chunk |
	[announcement displayProgressAt: Sensor cursorPoint
		from: 0 to: self size
		during:
		[:bar |
		[self atEnd]
			whileFalse: 
				[bar value: self position.
				self skipSeparators.
				[
					val _ (self peekFor: $!!) ifTrue: [
						(Compiler evaluate: self nextChunk logged: false) scanFrom: self
					] ifFalse: [
						chunk _ self nextChunk.
						self checkForPreamble: chunk.
						Compiler evaluate: chunk logged: aBoolean
					].
				]
					on: InMidstOfFileinNotification
					do: [ :ex | ex resume: true].
				self skipStyleChunk]]]
		ensure:[self close].
	"Note:  The main purpose of this banner is to flush the changes file."
	SmalltalkImage current logChange: '----End fileIn of ' , self name , '----'.
	SystemNavigation new allBehaviorsDo: [ :cl | 
		cl basicRemoveSelector: #DoIt; basicRemoveSelector: #DoItIn:
	].
	^ val! !

!MagmaRepositoryController methodsFor: '*magma tester' stamp: 'cmm 9/6/2007 21:44'!
getNextSequenceNumber
	^ self session commit: 
		[ (self session cacheAt: #testSequenceNumberGenerator)
			increment ;
			value ]! !

!MagmaRepositoryController methodsFor: '*magma tester' stamp: 'cmm 9/9/2007 21:30'!
initializeSequenceNumberGenerator
	"Have my session cache the MagmaCounter that will be accessed many hundreds of times (in a real-world app, that is) so performance is maximized."
	| sess counter |
	sess _ self session.
	counter _ MagmaCounter new.
	sess
		cacheAt: #testSequenceNumberGenerator
			put: counter ;
		commit: 
			[ sess root 
				at: #testSequenceNumberGenerator
				put: counter ].
	^ nil! !

MagmaNotification subclass: #MagmaTesterNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma tester-Notifications'!

!Class methodsFor: '*magma tester' stamp: 'cmm 8/17/2007 19:46'!
maRecursiveRemove
	self allSubclassesDo: [ : each | each maRecursiveRemove ].
	self removeFromSystem: false! !

MaNetworkTestCase subclass: #MagmaTestCase
	instanceVariableNames: 'magmaConsole peerConsole clientSession localRoot proxy afterImagesProgress original34990 forwardRecoverySignaled timeToVerifyRestore shouldStopGeneratingSequenceNumbers sequenceNumbers dormantObject objectToAdvance'
	classVariableNames: 'MagmaServerAddress InitiateBackupPrereqsCompleted VerifyRestorePreqreqsCompleted'
	poolDictionaries: ''
	category: 'Magma tester-SUnit tests'!

!MagmaTestCase commentStamp: '<historical>' prior: 0!
This class tests Magma functionality.
!

MagmaTestCase subclass: #MagmaCollectionTester
	instanceVariableNames: 'nonPersistentSamples samples samplesInMc fifthElement'
	classVariableNames: 'KeywordsSamples'
	poolDictionaries: ''
	category: 'Magma tester-SUnit tests'!

!MagmaCollectionTester class methodsFor: 'as yet unclassified' stamp: 'cmm 6/9/2004 14:13'!
initialize

	Smalltalk addToStartUpList: self! !

!MagmaCollectionTester class methodsFor: 'as yet unclassified' stamp: 'cmm 7/15/2008 16:29'!
startClient1Player
	self startClientPlayerOn: 1314! !

!MagmaCollectionTester class methodsFor: 'as yet unclassified' stamp: 'cmm 7/15/2008 16:29'!
startClient2Player
	self startClientPlayerOn: 1315! !

!MagmaCollectionTester class methodsFor: 'as yet unclassified' stamp: 'cmm 7/15/2008 16:29'!
startClientPlayerOn: portNumber
	| server testPlayer |
	self removeServerPackage.
	testPlayer := MaNetworkTestPlayer testCaseClass: self.
	server := MaTcpRequestServer protocol: testPlayer protocol.
	server console: MaTranscriptConsole new.
	server 
		processOn: portNumber
		using: [ : req | testPlayer processRequest: req ].
	server explore! !

!MagmaCollectionTester class methodsFor: 'as yet unclassified' stamp: 'cmm 4/27/2008 22:28'!
startServerPlayer
	| server testPlayer |
	MaObjectRepository runningTestCases: true.
	testPlayer := MaNetworkTestPlayer testCaseClass: self.
	server := MaTcpRequestServer protocol: testPlayer protocol.
	MagmaCollectionIndex maRecursiveRemove.
	server console: MaTranscriptConsole new.
	server 
		processOn: 1316
		using: [ : req | testPlayer processRequest: req ].
	{  server. testPlayer  } explore! !

!MagmaCollectionTester class methodsFor: 'as yet unclassified' stamp: 'cmm 8/19/2007 21:09'!
startUp: resuming 
	super startUp: resuming.
	resuming ifTrue: 
		[ self isClient1Image ifTrue: [ self startClient1Player ].
		self isClient2Image ifTrue: [ self startClient2Player ].
		self isServerImage ifTrue: [ self startServerPlayer ] ]! !

!MagmaCollectionTester methodsFor: 'test keyword index' stamp: 'cmm 4/14/2005 17:29'!
add: anAssociation to: aMagmaCollection smalltalkVersion: smalltalkIndex using: aMaKeywordIndexDefinition

	aMagmaCollection add: anAssociation.
	(aMaKeywordIndexDefinition indexObjectFor: anAssociation) do:
		[ : eachWord |
		| bag |
		bag _ 
		smalltalkIndex
			at: eachWord
			ifAbsentPut: [ nil "we just need the entry-key, bag must be indexed at a different key" ].
		"We never add anAssociation twice to the MagmaCollection, therefore we must be sure not to add it twice to the Smalltalk collection."
		bag _ 
			smalltalkIndex
				at: (aMaKeywordIndexDefinition indexHashForIndexObject: eachWord)
				ifAbsentPut: [ Bag new ].
		bag add: anAssociation ]! !

!MagmaCollectionTester methodsFor: 'private' stamp: 'cmm 6/5/2007 23:25'!
addAll: aCollection at: aString 
	self 
		remotePerform: #addAllToLocalCollection:at:
		withArguments: {  aCollection. aString  }
		in: 'client1'.
	^ self 
		remotePerform: #addAllToLargeCollection:at:
		withArguments: {  aCollection. aString  }
		in: 'client2'! !

!MagmaCollectionTester methodsFor: 'client' stamp: 'cmm 6/5/2007 00:17'!
addAllToLargeCollection: aCollection at: aString 
	| root |
	self 
		addAllToLocalCollection: aCollection
		at: aString.
	root _ clientSession root.
	clientSession commit: 
		[ aCollection do: 
			[ : each | 
			(root 
				at: aString
				ifAbsentPut: [ MagmaCollection new ]) add: each ] ].
	^ clientSession oidFor: (root at: aString)! !

!MagmaCollectionTester methodsFor: 'client' stamp: 'cmm 7/18/2002 15:40'!
addAllToLocalCollection: aCollection at: aString

	(localRoot
		at: aString
		ifAbsentPut: [ OrderedCollection new ])
			addAll: aCollection.! !

!MagmaCollectionTester methodsFor: 'test major functions' stamp: 'cmm 8/20/2006 21:39'!
addIndex
	clientSession commit:
		[ (clientSession root at: 'testMajorFunctions') addIndex:
			((MaSearchStringIndex attribute: #yourself)
				keySize: 64 ;
				beAscii ;
				yourself) ]! !

!MagmaCollectionTester methodsFor: 'test add index and objects simultaneously' stamp: 'cmm 8/17/2005 19:10'!
addIndexAndObjectsSimultaneously
	| c |
	c := MagmaCollection new
		addIndex: (MaKeywordIndex attribute: #last) ;
		yourself.
	(MaFixtureFactory new newPersonNames: 1) do:
		[ : each |
		c add: each ].
	clientSession commit:
		[ clientSession root 
			at: 'testAddIndexAndObjectsSimultaneously'
			put: c ]! !

!MagmaCollectionTester methodsFor: 'test keyword index' stamp: 'cmm 10/30/2004 17:56'!
addKeywordIndexAndRemainingClassesUsing: smalltalkIndex
	"I run in client2."

	| indexDef mc |
	indexDef _ self newKeywordIndex.
	mc _ clientSession root at: 'testKeywordIndex'.
	clientSession commit: [ mc addIndex: indexDef ].
	Collection withAllSubclassesDo:
		[ :each |
		clientSession commit:
			[ self notify: 'adding ', each name.
			self
				add: each maCommentKeywords -> each name
				to: mc
				smalltalkVersion: smalltalkIndex
				using: indexDef ] ].
	^smalltalkIndex! !

!MagmaCollectionTester methodsFor: 'test concurrency' stamp: 'cmm 8/14/2007 00:37'!
addObjectsForConcurrencyTest
	| oc mc |
	oc _ clientSession root at: 'testConcurrency_oc'.
	mc _ clientSession root at: 'testConcurrency_mc'.
	"Add the skipped element."
	clientSession commit: [ mc add: (oc at: 5) ]! !

!MagmaCollectionTester methodsFor: 'test querys - persistent populated' stamp: 'cmm 8/6/2006 23:08'!
addSomeRemoveSome: aCollectionOrMagmaCollection 
	"For the most-effective tests, we should combine all the most complicated scenarios."
	self initializeNonPersistentSamples.
	"add some non-persistent objects"
	aCollectionOrMagmaCollection addAll: nonPersistentSamples.
	"remove some persistent members"
	(1 
		to: samples size
		by: 29) do: [ : n | aCollectionOrMagmaCollection remove: (self sampleAt: n) ].
	"remove some of the non-persistent members (that we added in step1)"
	1 
		to: nonPersistentSamples size
		by: 29
		do: [ : n | aCollectionOrMagmaCollection remove: (nonPersistentSamples at: n) ]! !

!MagmaCollectionTester methodsFor: 'test backup/restore' stamp: 'cmm 6/23/2007 16:26'!
addVerifyRestorePrerequisite: aSymbol 
	self verifyRestorePreqreqsCompleted add: aSymbol! !

!MagmaCollectionTester methodsFor: 'private-querys tests' stamp: 'cmm 7/11/2006 23:23'!
assertMc: aMagmaCollectionReader equals: anOrderedCollection 
	| sc |
	[ aMagmaCollectionReader sortComplete ] whileFalse: 
		[ self notify: 'sort is ' , (aMagmaCollectionReader fractionSorted * 100 asFloat truncateTo: 1) asString , ' complete.'.
		(Delay forSeconds: 1) wait ].
	sc _ anOrderedCollection asSortedCollection: aMagmaCollectionReader sortBlock.
	self 
		verifyOrder: aMagmaCollectionReader
		against: sc! !

!MagmaCollectionTester methodsFor: 'private-querys tests' stamp: 'cmm 7/18/2006 22:56'!
assertMc: aMagmaCollectionReader includesAll: anOrderedCollection 
	"We cannot assume sizes will be equal because select: does not include element once for each qualifying clause like readers do.."
	self assert: ((aMagmaCollectionReader asArray: aMagmaCollectionReader size) maIncludesAll: anOrderedCollection)! !

!MagmaCollectionTester methodsFor: 'test backup/restore' stamp: 'cmm 6/24/2007 23:20'!
calculateTimeToVerifyRestore
	^ self requiredPrereqsToVerifyRestore = VerifyRestorePreqreqsCompleted! !

!MagmaCollectionTester methodsFor: 'test concurrency' stamp: 'cmm 8/18/2007 20:07'!
changeAddedObjects
	"If this doesn't signal a commit error, then verifyFifthElement better work."
	| commitConflicts |
	[ clientSession noteOldKeysFor: fifthElement.
	fifthElement key: 14.
	clientSession commit ] 
		on: MagmaCommitError
		do: [ : err | commitConflicts _ err result commitConflicts ].
	self assert: commitConflicts notNil.
	self assert: commitConflicts size = 1.
	self assert: commitConflicts anyOne conflictingObjects size = 1.
	self assert: commitConflicts anyOne conflictingObjects anyOne == fifthElement! !

!MagmaCollectionTester methodsFor: 'test MagmaArray' stamp: 'cmm 3/22/2005 17:15'!
changeAtIndex50

	clientSession commit:
		[ (clientSession root at: 'testMagmaArray')
			at: 50
			put: 'Fifty (50)' ]! !

!MagmaCollectionTester methodsFor: 'test major functions' stamp: 'cmm 6/15/2006 17:30'!
changeIndexKeysInClient1
	| reader changedKeys |
	reader _ (clientSession root at: 'testMajorFunctions') where: 
		[ : rdr | 
		rdr 
			read: #yourself
			from: '0'
			to: 'zzzzzzzzzzzzzzzzzzz' ].
	changedKeys _ OrderedCollection new.
	self notify: 'in changeIndexKeysInClient1'.
	self 
		sampling: 20
		in: reader
		do: 
			[ : eachKey | 
			changedKeys add: eachKey.
			self reverseString: ((localRoot at: 'testMajorFunctions') detect: [ : each | each = eachKey ]).
			clientSession
				noteOldKeysFor: eachKey ;
				commit: [ self reverseString: eachKey ] ].
	^ changedKeys! !

!MagmaCollectionTester methodsFor: 'test major functions' stamp: 'cmm 7/15/2006 12:19'!
changeIndexKeysInClient2
	| stringsChanged reader |
	stringsChanged _ OrderedCollection new.
	self notify: 'in changeIndexKeysInClient2'.
	reader _ (clientSession root at: 'testMajorFunctions') where: 
		[ : rdr | 
		rdr 
			read: #yourself
			from: '0'
			to: 'zzzzzzzzzzzzzzzzzzzzzzz' ].
	self 
		sampling: 10
		in: reader
		do: 
			[ : eachString | 
			clientSession
				noteOldKeysFor: eachString ;
				commit: [ self reverseString: eachString ].
			stringsChanged add: eachString ].
	^ stringsChanged! !

!MagmaCollectionTester methodsFor: 'test MagmaArray' stamp: 'cmm 3/22/2005 13:59'!
changeOthers

	| ma |
	ma _ clientSession root at: 'testMagmaArray'.
	1 to: 49 do: [ : n | ma at: n put: ((ma at: n) asNumber raisedTo: 2) ].
	51 to: 100 do: [ : n | ma at: n put: ((ma at: n) asNumber raisedTo: 2) ].
	clientSession commit.  "should be no commit-conflict because client1 only changed index 50."
	clientSession begin "prepare for next test"! !

!MagmaCollectionTester methodsFor: 'test MagmaArray' stamp: 'cmm 3/20/2005 21:41'!
changeSome

	clientSession commit:
		[ (clientSession root at: 'testMagmaArray')
			at: 50
			put: 'Fifty' ]! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 4/12/2005 17:41'!
commitMcForNpMcTest

	| mc |
	mc _ MagmaCollection new.
	mc
		add: 'one' ;
		add: 'two' ;
		add: 'three'.
	clientSession commit:
		[ clientSession root at: 'testNpIntegrationForMagmaCollections' put: mc ]! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 7/16/2006 13:05'!
commitReaderForNpIntegrationTest
	| mc reader |
	mc _ self newIndexedMagmaCollection.
	reader _ mc where: 
		[ : rdr | 
		rdr 
			read: #key
			from: 'c' "first in reader = charlie denise"
			to: 'oz' "last in reader = nancy oprah" ].
	reader pageSize: 2.	"test with pagesize smaller than sample-range."
	clientSession commit: 
		[ clientSession root 
			at: 'testNpIntegrationForReaders'
			put: reader ]! !

!MagmaCollectionTester methodsFor: 'test backup/restore' stamp: 'cmm 8/1/2007 22:16'!
copyCommitLogsToBackup
	| sourceDir targetDir |
	magmaConsole controller archiveCommitLog.
	sourceDir _ magmaConsole controller commitLogDirectory.
	targetDir _ (FileDirectory on: (self backupDirectory fullNameFor: MaRecoveryManager commitLogDirectoryName)) assureExistence.
	sourceDir fileNames do: 
		[ : each | 
		sourceDir 
			maCopyFileNamed: each
			toDirectory: targetDir ].
	^ magmaConsole controller commitNumber! !

!MagmaCollectionTester methodsFor: 'test MagmaSet' stamp: 'cmm 8/20/2006 21:39'!
createAndPopulateMagmaSet
	| newSet |
	clientSession commit: 
		[ clientSession root 
			at: 'testMagmaSet'
			put: (
				MagmaSet new
					addIndex: (MaSearchStringIndex attribute: #key) beAscii ;
					equivalenceAttributes: #(key) ;
					yourself) ].
	newSet _ clientSession root at: 'testMagmaSet'.
	self
		should: [ newSet isEmpty ] ;
		shouldnt: [ newSet includes: 3 -> 4 ].
	self 
		should: [ newSet add: Object new ]
		raise: MagmaUserError.
	self 
		should: [ newSet remove: 3 -> 4 ]
		raise: MagmaEquivalentObjectNotInCollection.
	self magmaSetEntries do: [ : eachAssociation | clientSession commit: [ newSet add: eachAssociation ] ]! !

!MagmaCollectionTester methodsFor: 'test persistent reader' stamp: 'cmm 7/19/2007 22:40'!
createCollectionAndReader
	| collection reader |
	collection _ (MagmaCollection new)
		addIndex: (MaByteSequenceIndex attribute: #key) ;
		yourself.
	reader _ collection where: 
		[ : rdr | 
		rdr 
			read: #key
			from: 'M'
			upTo: 'N' ].
	{  MaObject. Collection  } do: 
		[ : eachRoot | 
		eachRoot withAllSubclassesDo: 
			[ : each | 
			collection add: each name asString -> each hash	"meaningless for this test, other than let's not add the entire image to the class-definitions please" ] ].
	clientSession commit: 
		[ clientSession root 
			at: 'testPersistentReader.collection'
			put: collection.
		clientSession root 
			at: 'testPersistentReader.reader'
			put: reader ]! !

!MagmaCollectionTester methodsFor: 'test querys - persistent empty' stamp: 'cmm 7/16/2006 20:27'!
createEmpty
	clientSession commit: 
		[ clientSession root 
			at: 'testPersistentEmpty'
			put: self newMc ]! !

!MagmaCollectionTester methodsFor: 'test keyword index' stamp: 'cmm 10/30/2004 17:56'!
createInitialCollectionOfClasses
	"I'm running in client1"

	| mc answer indexDef root |
	mc _ MagmaCollection new.
	answer _ Dictionary new.
	indexDef _ self newKeywordIndex.
	root _ clientSession root.
	clientSession commit:
		[ root
			at: 'testKeywordIndex'
			put: mc.
		MaObject withAllSubclasses do:
			[ : each |
			self
				add: each maCommentKeywords -> each name
				to: mc
				smalltalkVersion: answer
				using: indexDef ] ].
	^ answer! !

!MagmaCollectionTester methodsFor: 'test querys - persistent result sets' stamp: 'cmm 10/8/2006 15:54'!
createInitialIndexedCollection
	| mc array |
	mc _ (MagmaCollection new)
		addIndex: (MaIntegerIndex attribute: #first) ;
		yourself.
	"Array of 1-element Array's."
	array _ (1 to: 100) collect: [ : n | Array with: n ].
	mc addAll: array.
	"add some duplicates"
	(array 
		copyFrom: 51
		to: 60) do: [ : each | mc add: each ].
	clientSession commit: 
		[ clientSession root 
			at: 'testPersistentResultSet-originalCollection'
			put: mc ]! !

!MagmaCollectionTester methodsFor: 'test MagmaArray' stamp: 'cmm 3/20/2005 21:37'!
createMagmaArray

	| ma |
	ma _ MagmaArray new.
	1 to: 100 do: [ : n | ma at: n put: n printString ].
	clientSession commit:
		[ clientSession root
			at: 'testMagmaArray'
			put: ma ]! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 7/16/2006 13:00'!
createMagmaArrayForNpTest
	| arr |
	arr _ MagmaArray new.
	arr at: 5 put: 'five'.
	clientSession commit:
		[ clientSession root 
			at: 'testNpIntegrationForMagmaArrays'
			put: arr ]! !

!MagmaCollectionTester methodsFor: 'test stale collections' stamp: 'cmm 12/29/2004 09:53'!
createMagmaCollection

	| mc |
	mc _ MagmaCollection new.
	MaTestCase withAllSubclasses do: [ : each | mc add: each ].
	clientSession commit:
		[ clientSession root
			at: 'testStaleCollections'
			put: mc ].
	localRoot
		at: 'testStaleCollections'
		put: mc! !

!MagmaCollectionTester methodsFor: 'test querys - persistent populated' stamp: 'cmm 8/6/2006 23:33'!
createPersistentPopulated
	| mc |
	self initializeSamples.
	mc _ self newMc.
	clientSession commit: 
		[ clientSession root 
			at: 'testPersistentPopulated'
			put: mc ].
	clientSession commit: [ self load: mc ].
	samplesInMc _ samples copy asSet! !

!MagmaCollectionTester methodsFor: 'test querys - persistent result sets' stamp: 'cmm 11/9/2006 18:13'!
createTransientResultSetAndVerify
	| sourceMc firstReader firstMc newReader secondReader |
	clientSession abort.
	sourceMc := clientSession root at: 'testPersistentResultSet-originalCollection'.

	"Note this is the non-blocking where.."
	firstReader := sourceMc 
		where: 
			[ : rdr | 
			rdr 
				read: #first
				from: 41
				to: 70 ]
		distinct: true
		sortBy: #first
		descending: false.

	"And, immediately, begin a (blocking) query where it creates, yet, another new non-persistent subcollection off the first non-persistent collection!!"
	secondReader := firstReader 
		where: 
			[ : rdr | 
			rdr 
				read: #first
				from: 1
				to: 50 ]
		distinct: true
		sortedBy: #first
		descending: true.
	self assert: secondReader size = 10.
	self assert: secondReader collection size = 10.
	self assert: secondReader first = {  50  }.
	self assert: secondReader last = {  41  }.
	self deny: (clientSession isPersistent: secondReader).

	"Do some queries against the first reader."
	firstMc := firstReader collection.
	self assert: firstReader size = 30.
	self assert: firstMc ~~ sourceMc.
	self assert: firstMc size = 30.

	"an additional where query (which won't create a new underlying collection) against the first non-persistent collection."
	newReader := firstReader where: 
		[ : rdr | 
		rdr 
			read: #first
			from: 45
			to: 54 ].
	self assert: secondReader collection ~~ newReader collection.
	self assert: newReader size = 10.
	self assert: newReader first = {  45  }.
	self assert: newReader last = {  54  }.

	"Now persist the second result-set so client1 can do some assertions."
	clientSession commit: 
		[ clientSession root 
			at: 'testPersistentResultSet-secondReader'
			put: secondReader ].

	self assert: (clientSession isPersistent: secondReader)! !

!MagmaCollectionTester methodsFor: 'test querys - persistent populated' stamp: 'cmm 8/6/2006 23:33'!
dressUpAndVerify
	| mc oc |
	mc _ clientSession root at: 'testPersistentPopulated'.
	samplesInMc _ (mc asArray: mc size) asSet.
	self
		initializeSamples ;
		initializeNonPersistentSamples.
	samplesInMc addAll: nonPersistentSamples.
	oc _ OrderedCollection new: samples size.
	self
		load: oc ;
		addSomeRemoveSome: mc;
		addSomeRemoveSome: oc.
	"changeSomeKeysIn: mc do: [ : each | clientSession noteOldKeysFor: each ] ;
		changeSomeKeysIn: oc do: [ : each | ];"
	self 
		verifyMc: mc
		against: oc! !

!MagmaCollectionTester methodsFor: 'test concurrency' stamp: 'cmm 8/14/2007 21:11'!
initializeIndexedCollectionForConcurrencyTest
	| mc oc |
	oc _ OrderedCollection new.
	mc _ MagmaCollection new
		addIndex: (MaIntegerIndex attribute: #key) ;
		yourself.
	1 
		to: 500
		by: 3
		do: 
			[ : n | 
			n -> n asWords in: 
				[ : obj | 
				"skip the fifth element for the MagmaCollection, we'll add it later."
				n = 13 ifFalse: [ mc add: obj ].
				oc add: obj ] ].
	clientSession commit: 
		[ clientSession root
			at: 'testConcurrency_mc'
				put: mc ;
			at: 'testConcurrency_oc'
				put: oc ].
	fifthElement := oc at: 5.
	clientSession begin! !

!MagmaCollectionTester methodsFor: 'private-samples' stamp: 'cmm 8/12/2007 20:29'!
initializeNonPersistentSamples
	"Lets have 100 non-persistent samples interleaved with the persistent ones.."
	| firsts seconds thirds r |
	nonPersistentSamples _ OrderedCollection new.
	firsts _ (5 
		to: 1000
		by: 10) asArray shuffledBy: self random.
	seconds _ ((2453905 
		to: 2454895
		by: 10) collect: [ : e | Date julianDayNumber: e ]) shuffledBy: self random.
	r _ self random.
	thirds _ (1 to: 100) collect: [ : n | (r nextInt: (1 bitShift: 128)) asWords subStrings ].
	1 to: firsts size do: 
		[ : n | 
		nonPersistentSamples add: {  (firsts at: n). (seconds at: n). (thirds at: n)  } ]! !

!MagmaCollectionTester methodsFor: 'private-samples' stamp: 'cmm 7/30/2006 16:45'!
initializeSamples
	"Build a collection of four-element Array's."
	| firsts seconds thirds r|
	r _ self random.
	samples _ OrderedCollection new: 1000.
	firsts _ (1 
		to: 10000
		by: 10) asArray shuffledBy: r.
	seconds _ ((2453900 
		to: 2463899
		by: 10) collect: [ : e | Date julianDayNumber: e ]) shuffledBy: r.
	thirds _ self keywordsSamples.
	1 
		to: firsts size
		do: 
			[ : n | 
			samples add: { 
					(firsts at: n).
					(seconds at: n).
					(thirds at: n)
				 } ]! !

!MagmaCollectionTester methodsFor: 'private-samples' stamp: 'cmm 7/30/2007 23:44'!
keywordsSamples
	^ KeywordsSamples ifNil: 
		[ | cmInstances |
		KeywordsSamples _ OrderedCollection new.
		cmInstances _ CompiledMethod allInstances readStream.
		[ cmInstances atEnd or: [ KeywordsSamples size >= 1000 ] ] whileFalse: 
			[ | nextInstance |
			nextInstance _ cmInstances next.
			nextInstance who first = #unknown ifFalse: 
				[ KeywordsSamples add: nextInstance maSourceString subStrings asSet asArray ] ].
		self assert: KeywordsSamples size = 1000.
		KeywordsSamples ]! !

!MagmaCollectionTester methodsFor: 'private-querys tests' stamp: 'cmm 7/11/2006 23:23'!
load: aCollection 
	self 
		load: aCollection
		from: 1
		to: samples size! !

!MagmaCollectionTester methodsFor: 'private-querys tests' stamp: 'cmm 7/11/2006 23:24'!
load: aCollection from: startIndex to: endIndex
	startIndex to: endIndex do:
		[ : x |
		aCollection add: (samples at: x) ]! !

!MagmaCollectionTester methodsFor: 'test MagmaSet' stamp: 'cmm 2/2/2005 00:22'!
magmaSetEntries

	^ { 'one'->1.
	'two'->2.
	'three'->3 }! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 2/19/2008 12:56'!
newIndexedMagmaCollection

	| mc |
	mc := MagmaCollection new
		addIndex: (MaKeywordIndex attribute: #key) ;
		yourself.
	{ 'alpha beta'.
	'charlie denise'.
	'edward frank'.
	'tom dick harry'.
	'jack karen'.
	'larry greg'.
	'nancy ozzie'.
	'peter queso'.
	'randy steve'.
	'tina uma'.
	'vivian wilbur'.
	'xavier yo-yo-ma'.
	'zebra zzzzzzz' } do: [ : each | mc add: each substrings -> Object new ].
	^ mc! !

!MagmaCollectionTester methodsFor: 'private-querys tests' stamp: 'cmm 7/11/2006 23:23'!
newMc
	"Create a new MagmaCollection with the four indexes used for all the tests."
	^ MagmaCollection new
		addIndex: (MaIntegerIndex attribute: #first) ;
		addIndex: (MaDateIndex attribute: #second) ;
		addIndex: (MaKeywordIndex attribute: #third) ;
		yourself! !

!MagmaCollectionTester methodsFor: 'test backup/restore' stamp: 'cmm 6/24/2007 23:23'!
noteVerifyPersistentResultSetComplete
	self addVerifyRestorePrerequisite: #verifyPersistentResultSet.
	^ self calculateTimeToVerifyRestore ! !

!MagmaCollectionTester methodsFor: 'private-samples' stamp: 'cmm 7/11/2006 23:25'!
persistentSamples
	^ (1 to: 100) collect: [ : n | self sampleAt: n ]! !

!MagmaCollectionTester methodsFor: 'private' stamp: 'cmm 8/23/2007 16:56'!
playerProtocol

	^ super playerProtocol
		addAll: { Bag.  Association. True. False } ;
		yourself! !

!MagmaCollectionTester methodsFor: 'private-querys tests' stamp: 'cmm 7/11/2006 23:23'!
random
	"Use a specific random so we get consistent results."
	^ Random seed: 51969! !

!MagmaCollectionTester methodsFor: 'test stale collections' stamp: 'cmm 12/28/2004 13:16'!
readMagmaCollection

	clientSession abort.
	localRoot
		at: 'testStaleCollections'
		put: (clientSession root at: 'testStaleCollections')! !

!MagmaCollectionTester methodsFor: 'test major functions' stamp: 'cmm 11/23/2002 16:37'!
removeFromLocalRoot: aCollection at: aString

	(localRoot at: aString) removeAll: aCollection! !

!MagmaCollectionTester methodsFor: 'test major functions' stamp: 'cmm 8/17/2005 19:33'!
removeIndex

	| mc |
	clientSession abort.
	mc := clientSession root at: 'testMajorFunctions'.
	clientSession commit: [ mc removeIndexNamed: #yourself ]! !

!MagmaCollectionTester methodsFor: 'test major functions' stamp: 'cmm 2/20/2008 12:22'!
removeSomeEntriesInClient2Including: reversedString 
	"Remove reversedString, as well as 20 additional entries using two different methods."
	| stringsRemoved reader localCollection priorReaderSize root |
	root _ clientSession root.
	localCollection _ localRoot at: 'testMajorFunctions'.
	"First, remove reversedString."
	stringsRemoved _ OrderedCollection new.
	reader _ (root at: 'testMajorFunctions') where: 
		[ : rdr | 
		rdr 
			read: #yourself
			at: reversedString ].
	clientSession abort.
	self should: 
		[ (reader asArray: 10) allSatisfy: 
			[ : eachString | 
			(eachString 
				copyFrom: 1
				to: (reader sortIndex meaningfulCharacters min: eachString size)) = (reversedString 
					copyFrom: 1
					to: (reader sortIndex meaningfulCharacters min: reversedString size)) ] ].
	priorReaderSize _ reader size.
	stringsRemoved add: reader first.
	localCollection remove: reader first.
	clientSession commit: [ (root at: 'testMajorFunctions') remove: reader first ].
	self should: [ reader refresh size = (priorReaderSize - 1) ].
	reader _ (root at: 'testMajorFunctions') read: #yourself.

	"Remove 20 random entries, committing each time."
	self 
		sampling: 20
		in: localCollection copy
		do: 
			[ : objectToRemove | 
			stringsRemoved add: objectToRemove.
			clientSession commit: [ (root at: 'testMajorFunctions') remove: objectToRemove ].
			localCollection remove: objectToRemove ].

	"Remove another 20 with a single commit."
	clientSession commit: 
		[ self 
			sampling: 20
			in: localCollection copy
			do: 
				[ : objectToRemove | 
				stringsRemoved add: objectToRemove.
				(root at: 'testMajorFunctions') remove: objectToRemove.
				localCollection remove: objectToRemove ] ].
	^ stringsRemoved! !

!MagmaCollectionTester methodsFor: 'test backup/restore' stamp: 'cmm 7/1/2007 21:58'!
requiredPrereqsToVerifyRestore
	"Answer the tests which must have completed before verifyRestore can be run."
	^ #(
		#verifyCollectionAndReader
		#verifyAddedCollectionAndIndex
		#verifyPersistentResultSet
	) asSet! !

!MagmaCollectionTester methodsFor: 'test backup/restore' stamp: 'cmm 6/21/2007 21:18'!
restoreVerifications
	"Answer the selectors of the verification methods we will use to ensure the restored backup looks equivalent to the original."
	^ self requiredPrereqsToVerifyRestore , #(
			#verifySamples
			#verifyTestSet
			#verifyDoubleUpdate
			#verifyMagmaCollectionAndIndexCreatedSimultaneously
			#verifyUpdateByteObjects
			#readAndVerifyGrownObject
			#verifyReferenceToSession
		)! !

!MagmaCollectionTester methodsFor: 'test major functions' stamp: 'cmm 11/25/2002 01:06'!
reverseLocalStrings: aCollection
	"aCollection of strings that are already reversed, need to replace their normal counterparts in my local collection."

	aCollection do:
		[ :e |
		| reversee |
		(localRoot at: 'testMajorFunctions')
			remove: e reverse;
			add: e ]! !

!MagmaCollectionTester methodsFor: 'private' stamp: 'cmm 10/4/2004 16:44'!
reverseString: aString
	self notify: 'reversing ', aString.
	1 to: aString size // 2 do:
		[ :eachPosition |
		aString 
			swap: eachPosition 
			with: (aString size - eachPosition + 1) ]! !

!MagmaCollectionTester methodsFor: 'private-samples' stamp: 'cmm 8/6/2006 23:08'!
sampleAt: indexInteger 
	^ samplesInMc detect: [ : e | e = (samples at: indexInteger) ]! !

!MagmaCollectionTester methodsFor: 'private-samples' stamp: 'cmm 7/11/2006 23:25'!
samplesOf: attribute
	^ samples collect: [ : e | e perform: attribute ]! !

!MagmaCollectionTester methodsFor: 'private' stamp: 'cmm 10/6/2004 15:54'!
sampling: count in: aMagmaCollectionReader do: oneArgBlock

	| increment |
	increment _ (aMagmaCollectionReader size / 2 / count) asInteger max: 1.
	1 
		to: aMagmaCollectionReader size // 2
		by: increment
		do:
			[ : x |
			oneArgBlock value: (aMagmaCollectionReader at: x) ]! !

!MagmaCollectionTester methodsFor: 'testMultiKeywordMatching' stamp: 'cmm 7/5/2008 20:18'!
setUpTestMultiKeywordMatching
	clientSession
		abort ;
		commit: 
			[ | c |
			clientSession root 
				at: #testMultiKeywordMatching
				put: (c := MagmaCollection new
						addIndex: self newKeywordIndex ;
						yourself).
			1 
				to: 1000
				do: [ : n | c add: n asWords subStrings -> n ] ]! !

!MagmaCollectionTester methodsFor: 'test add index and objects simultaneously' stamp: 'cmm 6/24/2007 23:22'!
testAddIndexAndObjectsSimultaneously
	"There was a bug once where adding an new index along with new objects would cause the new objects to be indexed twice.  This ensures that remains fixed."
	self 
		remotePerform: #addIndexAndObjectsSimultaneously
		in: 'client1'.
	timeToVerifyRestore _ self 
		remotePerform: #verifyAddedCollectionAndIndex
		in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test concurrency' stamp: 'cmm 8/14/2007 21:12'!
testConcurrency
	"This tests the ability of Magma to correctly detect a commit conflict when an already-persistent object is added or removed, and another client then changes its indexed values."
	self 
		remotePerform: #initializeIndexedCollectionForConcurrencyTest in: 'client1' ;
		remotePerform: #addObjectsForConcurrencyTest in: 'client2' ;
		remotePerform: #changeAddedObjects in: 'client1' ;
		remotePerform: #verifyFifthElement in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test keyword index' stamp: 'cmm 10/6/2004 16:59'!
testKeywordIndex
	"Build a MagmaCollection of classes in the system, then add a keyword index by words in their class comments.  Build it as a Smalltalk structure as well, and use it to compare."

	| smalltalkVersion |
	smalltalkVersion _ self
		remotePerform: #createInitialCollectionOfClasses
		in: 'client1'.
	smalltalkVersion _ self
		remotePerform: #addKeywordIndexAndRemainingClassesUsing:
		with: smalltalkVersion
		in: 'client2'.
	self
		remotePerform: #verifyKeywordIndexUsing:
		with: smalltalkVersion
		in: 'client1'! !

!MagmaCollectionTester methodsFor: 'test MagmaArray' stamp: 'cmm 3/20/2005 21:55'!
testMagmaArray

	self
		remotePerform: #createMagmaArray in: 'client1' ;
		remotePerform: #verifyMagmaArray in: 'client2' ;
		remotePerform: #changeSome in: 'client1' ;
		remotePerform: #changeOthers in: 'client2' ;
		remotePerform: #changeAtIndex50 in: 'client1' ;
		remotePerform: #tryChangingIndex50 in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test MagmaSet' stamp: 'cmm 8/19/2007 11:54'!
testMagmaSet
	self
		remotePerform: #createAndPopulateMagmaSet
		in: 'client1'.
	self
		remotePerform: #verifyMagmaSet
		in: 'client2'.
	self
		remotePerform: #verifyReducedMagmaSet
		in: 'client1'.
! !

!MagmaCollectionTester methodsFor: 'test major functions' stamp: 'cmm 6/6/2007 20:41'!
testMajorFunctions
	"This test builds a local Smalltalk collection and a MagmaCollection in parallel.  The local is built in client1.  Most of the MagmaCollection is built in client2.  At the end, client1 verifies the MagmaCollection against the local."
	| tempSamples sample1 sample2 changedIndexKeys localEntriesToRemove stringsToReverse collectionOid |
	tempSamples _ (MaObject withAllSubclasses collect: [ : each | each name asString ]) asArray.
	sample1 _ tempSamples 
		copyFrom: 1
		to: tempSamples size // 2.
	sample2 _ tempSamples 
		copyFrom: tempSamples size // 2 + 1
		to: tempSamples size.
	"client2 creates the collection, adds some objects."
	collectionOid _ self 
		addAll: sample1
		at: 'testMajorFunctions'.
	"client1 adds an index."
	self 
		remotePerform: #addIndex
		in: 'client1'.
	"now client2 resumes adding with sample2."
	self 
		addAll: sample2
		at: 'testMajorFunctions'.
	changedIndexKeys _ self 
		remotePerform: #changeIndexKeysInClient1
		in: 'client1'.
	localEntriesToRemove _ self 
		remotePerform: #removeSomeEntriesInClient2Including:
		with: changedIndexKeys first
		in: 'client2'.
	self 
		remotePerform: #removeFromLocalRoot:at:
		withArguments: {  localEntriesToRemove. 'testMajorFunctions'  }
		in: 'client1'.
	stringsToReverse _ self 
		remotePerform: #changeIndexKeysInClient2
		in: 'client2'.
	self 
		remotePerform: #reverseLocalStrings:
		with: stringsToReverse
		in: 'client1'.
	self 
		remotePerform: #verifyReaderMatchesLocalAt:on:
		withArguments: #('testMajorFunctions' #yourself )
		in: 'client1'.
	self 
		remotePerform: #verifyIndexFilesFor:present:
		withArguments: {collectionOid. true}
		in: 'server'.
	self 
		remotePerform: #removeIndex
		in: 'client2'.
	"verify the actual collection in client1."
	self 
		remotePerform: #verifyLargeCollectionAt:
		with: 'testMajorFunctions'
		in: 'client2'.
	"Make sure caches flushed."
	(Delay forSeconds: 5) wait.
	self 
		remotePerform: #verifyIndexFilesFor:present:
		withArguments: {collectionOid. false}
		in: 'server'.! !

!MagmaCollectionTester methodsFor: 'testMultiKeywordMatching' stamp: 'cmm 7/5/2008 19:35'!
testMultiKeywordMatching
	self
		remotePerform: #setUpTestMultiKeywordMatching
			in: 'client1' ;
		remotePerform: #verifyMultiKeywordQueries
			in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test querys - non-persistent empty' stamp: 'cmm 7/11/2006 23:22'!
testNonPersistentEmpty
	| mc oc |
	mc _ self newMc.
	oc _ OrderedCollection new.
	self 
		verifyMc: mc
		against: oc! !

!MagmaCollectionTester methodsFor: 'test querys - non-persistent populated' stamp: 'cmm 8/7/2006 00:02'!
testNonPersistentPopulated
	| mc oc |
	self initializeSamples.
	mc _ self newMc.
	self load: mc.
	samplesInMc _ (mc asArray: mc size) asSet.
	oc _ OrderedCollection new: samples size.
	"Make a new copy of the samples because dressUp will modify the objects in-place, we want to do that independently for the mc and oc."
	self initializeSamples.
	self load: oc.
	self 
		verifyMc: mc
		against: oc.
	self
		addSomeRemoveSome: mc ;
		addSomeRemoveSome: oc.
	self 
		verifyMc: mc
		against: oc! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 7/15/2006 15:19'!
testNonPersistentReader
	| mc reader |
	mc _ self newIndexedMagmaCollection.
	reader _ mc where: 
		[ : rdr | 
		rdr 
			read: #key
			from: 'c'
			to: 'oz' ].
	reader pageSize: 2.	"test with pagesize smaller than sample-range."
	self verifyReaderForNpIntegrationTest: reader! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 4/12/2005 17:19'!
testNpIntegrationForMagmaArrays

	| arr |
	"First, test a completely non-persistent MagmaArray."
	arr _ MagmaArray new.
	arr at: 5 put: 'five'.
	self should: [ arr size = 5 ].
	self should: [ (arr at: 5) = 'five' ].
	self should: [ (arr at: 1) = nil ].
	self
		should: [ arr at: 6 ]
		raise: Error.
	self
		remotePerform: #createMagmaArrayForNpTest in: 'client1' ;
		remotePerform: #verifyMagmaArrayForNpTest in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 4/12/2005 17:30'!
testNpIntegrationForMagmaCollections

	self
		remotePerform: #commitMcForNpMcTest in: 'client1' ;
		remotePerform: #verifyMcForNpMcTest in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 4/12/2005 17:37'!
testNpIntegrationForReaders

	self
		remotePerform: #commitReaderForNpIntegrationTest in: 'client1' ;
		remotePerform: #verifyReaderForNpIntegrationTest in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test querys - persistent empty' stamp: 'cmm 7/1/2007 21:59'!
testPersistentEmpty
	self 
		remotePerform: #createEmpty
		in: 'client1'.
	self 
		remotePerform: #verifyEmpty
		in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test querys - persistent populated' stamp: 'cmm 7/29/2006 16:49'!
testPersistentPopulated
	self
		remotePerform: #createPersistentPopulated
			in: 'client1' ;
		remotePerform: #dressUpAndVerify
			in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test persistent reader' stamp: 'cmm 6/24/2007 23:22'!
testPersistentReader
	self 
		remotePerform: #createCollectionAndReader
		in: 'client1'.
	timeToVerifyRestore _ self 
		remotePerform: #verifyCollectionAndReader
		in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test querys - persistent result sets' stamp: 'cmm 6/24/2007 23:22'!
testPersistentResultSet
	"When creating a distinct result-set, we have to create a file on the server.  But that file must have a unique-name, not a transient-oid name, and the user could make it persistent anyway.  Be sure this works."
	self 
		remotePerform: #createInitialIndexedCollection
		in: 'client1'.
	self 
		remotePerform: #createTransientResultSetAndVerify
		in: 'client2'.
	self 
		remotePerform: #verifyPersistentResultSet
		in: 'client1'.
	timeToVerifyRestore _ self 
		remotePerform: #noteVerifyPersistentResultSetComplete
		in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test stale collections' stamp: 'cmm 2/13/2008 21:10'!
testStaleCollections
	self
		remotePerform: #createMagmaCollection
		in: 'client1'.
	self
		remotePerform: #readMagmaCollection
		in: 'client2'.
	self disconnectClients.
	self
		remotePerform: #verifyMagmaCollection
		in: 'client1'.
	self
		remotePerform: #verifyMagmaCollection
		in: 'client2'! !

!MagmaCollectionTester methodsFor: 'test MagmaArray' stamp: 'cmm 2/12/2008 21:00'!
tryChangingIndex50

	| ma |
	ma _ clientSession root at: 'testMagmaArray'.
	1 to: 50 do: [ : n | ma at: n put: ('this string should not be persistent!!') ].
	self should:
		[ [ clientSession commit.  false ]
			on: MagmaCommitError
			do:
				[ : err |
				self should: [ err result commitConflicts size = 1 ].
				self should: [ err result commitConflicts first conflictingConnection userId = 'client1' ].
				self should: [ err result commitConflicts first conflictingObjects size = 1 ].
				self should: [ err result commitConflicts first conflictingObjects includes: ma ].
				true ] ]! !

!MagmaCollectionTester methodsFor: 'test add index and objects simultaneously' stamp: 'cmm 6/24/2007 23:23'!
verifyAddedCollectionAndIndex
	| reader |
	reader _ (clientSession root at: 'testAddIndexAndObjectsSimultaneously') where: 
		[ : r | 
		r 
			read: #last
			from: 'A' ].
	self should: [ reader size = 1 ].
	self addVerifyRestorePrerequisite: #verifyAddedCollectionAndIndex.
	^ self calculateTimeToVerifyRestore ! !

!MagmaCollectionTester methodsFor: 'test persistent reader' stamp: 'cmm 7/19/2007 23:15'!
verifyCollectionAndReader
	| reader |
	clientSession abort.
	reader _ clientSession root at: 'testPersistentReader.reader'.
	self assert: reader notEmpty.
	self should: [ reader allSatisfy: [ : eachName | eachName key first = $M ] ].
	self addVerifyRestorePrerequisite: #verifyCollectionAndReader.
	^ self calculateTimeToVerifyRestore ! !

!MagmaCollectionTester methodsFor: 'test querys - persistent empty' stamp: 'cmm 7/1/2007 21:59'!
verifyEmpty
	| mc |
	clientSession abort.
	mc _ clientSession root at: 'testPersistentEmpty'.
	self 
		verifyMc: mc
		against: OrderedCollection new! !

!MagmaCollectionTester methodsFor: 'test concurrency' stamp: 'cmm 8/18/2007 20:07'!
verifyFifthElement
	| mc reader |
	mc := clientSession abort root at: 'testConcurrency_mc'.
	reader := mc where: [ : each | each key = 14 ].
	self assert: reader isEmpty.
	reader := mc where: [ : each | each key = 13 ].
	self assert: reader size = 1! !

!MagmaCollectionTester methodsFor: 'test major functions' stamp: 'cmm 6/6/2007 20:37'!
verifyIndexFilesFor: collectionOid present: aBoolean 
	| description filename fact parts |
	description _ MagmaIndexDescription 
		attribute: #yourself
		keySize: 64
		recordSize: 8.
	filename _ (magmaConsole controller repository collectionManagerFor: collectionOid) 
		filenameFor: 'oids'
		of: description.
	parts _ (FileDirectory on: self environmentPath) splitNameVersionExtensionFor: filename.
	filename _ parts first, '.1.', parts last.
	fact _ magmaConsole controller repository directory fileExists: filename.
	aBoolean 
		ifTrue: [ self assert: fact ]
		ifFalse: [ self deny: fact ]! !

!MagmaCollectionTester methodsFor: 'test keyword index' stamp: 'cmm 6/15/2006 17:22'!
verifyKeywordIndexUsing: smalltalkIndex 
	"smalltalkIndex is a Dictionary of Bags.  Keys of the dictionary are the hash keys of the various keywords.  I run in client1."
	| mc index |
	clientSession abort.
	mc _ clientSession root at: 'testKeywordIndex'.
	index _ mc indexNamed: #key.
	smalltalkIndex keysAndValuesDo: 
		[ : eachWordOrHashInt : eachBag | 
		eachWordOrHashInt isString ifTrue: 
			[ | reader mcBag stBag |
			self notify: 'verifying keyword ' , eachWordOrHashInt.
			reader _ mc where: 
				[ : rdr | 
				rdr 
					read: #key
					at: eachWordOrHashInt ].
			mcBag _ (reader asArray: reader size) asBag.
			stBag _ smalltalkIndex at: (index indexHashForIndexObject: eachWordOrHashInt).
			self should: [ mcBag = stBag ] ] ]! !

!MagmaCollectionTester methodsFor: 'client' stamp: 'cmm 12/7/2004 14:49'!
verifyLargeCollectionAt: aString

	| localCollection largeCollection |

	clientSession abort.
	localCollection _
		localRoot 
			at: aString
			ifAbsentPut: [ OrderedCollection new ].
	largeCollection _ self clientSession root at: aString.
	self should: [ largeCollection size = localCollection size ].
	largeCollection do: [ :each | self should: [ localCollection includes: each ] ].
	localCollection do: [ :each | self should: [ (largeCollection includes: each) and: [ (largeCollection occurrencesOf: each) = 1 ] ] ]! !

!MagmaCollectionTester methodsFor: 'test MagmaArray' stamp: 'cmm 3/20/2005 21:42'!
verifyMagmaArray

	| ma |
	clientSession abort.
	ma _ clientSession root at: 'testMagmaArray'.
	1 to: 100 do:
		[ : n | 
		self should: [ (ma at: n) = n printString ].
		self shouldnt: [ (ma at: n) == n printString ] ].
	clientSession begin! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 3/20/2007 22:00'!
verifyMagmaArrayForNpTest
	| arr |
	arr _ clientSession root at: 'testNpIntegrationForMagmaArrays'.
	arr 
		at: 1
		put: 'one'.
	self should: [ arr size = 5 ].
	arr 
		at: 501
		put: 'five-hundred, one'.
	self should: [ arr size = 501 ].
	self should: [ (arr select: [ : e | e notNil ]) size = 3 ].
	6 
		to: 500
		do: [ : n | self should: [ (arr at: n) = nil ] ]! !

!MagmaCollectionTester methodsFor: 'test stale collections' stamp: 'cmm 8/17/2008 23:37'!
verifyMagmaCollection
	Smalltalk garbageCollect.
	self assert: (localRoot at: 'testStaleCollections') asArray asSet = MaTestCase withAllSubclasses! !

!MagmaCollectionTester methodsFor: 'test MagmaSet' stamp: 'cmm 7/12/2006 21:52'!
verifyMagmaSet

	| ms newEntries |
	ms _ clientSession root at: 'testMagmaSet'.
	newEntries _ self magmaSetEntries.
	self should: [ ms size = newEntries size ].
	newEntries do:
		[ : each |
		self should: [ ms includes: each ] ].
	self shouldnt: [ ms includes: 'three'-> 'different value' ].
	self shouldnt: [ ms includes: 'four'->4 ].
	self shouldnt: [ ms includes: Object new ].
	self
		should: [ ms add: 'one'->1 ]
		raise: MagmaDuplicateObjectInCollection.
	"Do a simple remove test too."
	newEntries do:
		[ : each | 
		clientSession commit: [ ms remove: each ] ].
	self should: [ ms isEmpty ]! !

!MagmaCollectionTester methodsFor: 'private-querys tests' stamp: 'cmm 7/18/2006 22:56'!
verifyMc: aMagmaCollection against: anOrderedCollection 
	self 
		assertMc: aMagmaCollection
		includesAll: anOrderedCollection.
	self whereAndSelectBlockPairs do: 
		[ : eachPair | 
		self 
			verifyWhere: eachPair first
			from: aMagmaCollection
			withSelect: eachPair second
			from: anOrderedCollection ]! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 4/14/2005 09:53'!
verifyMcForNpMcTest

	| mc |
	mc _ clientSession root at: 'testNpIntegrationForMagmaCollections'.
	mc add: 'four'; add: 'five'.
	self should: [ mc size = 5 ].
	self should: [ (mc asArray: 5) asSet = #('one' 'two' 'three' 'four' 'five') asSet ]! !

!MagmaCollectionTester methodsFor: 'testMultiKeywordMatching' stamp: 'cmm 7/5/2008 21:05'!
verifyMultiKeywordQueries
	| mc mcBeginsWithQueryResult smalltalkBeginsWithQueryResult mcEntireWordQueryResults smalltalkEntireWordQueryResult index |
	clientSession abort.
	mc := clientSession root at: #testMultiKeywordMatching.
	index := mc indexes first.
	mcBeginsWithQueryResult := (mc where: [ : each | each key includesAllPrefixes: #('th' 'fo' ) ]) asArray asBag.
	smalltalkBeginsWithQueryResult := ((1 to: 1000) collect: [ : n | n asWords subStrings -> n ]) asBag select: 
		[ : eachAssoc | 
		#('th' 'fo' ) allSatisfy: 
			[ : eachPrefix | 
			eachAssoc key anySatisfy: 
				[ : eachString | 
				eachString 
					maBeginsWith: eachPrefix
					caseSensitive: false ] ] ].
	"Compare Bags because consistent order is not required."
	self assert: mcBeginsWithQueryResult = smalltalkBeginsWithQueryResult.
	"Now ensure entire keywords search works equally well."
	mcEntireWordQueryResults := (mc where: [ : each | each key includesAllOf: #('three' 'seven' ) ]) asArray asBag.
	smalltalkEntireWordQueryResult := ((1 to: 1000) collect: [ : n | n asWords subStrings -> n ]) asBag select: 
		[ : eachAssoc | 
		#('three' 'seven' ) allSatisfy: 
			[ : eachPrefix | 
			eachAssoc key anySatisfy: 
				[ : eachString | 
				(eachString first: (index meaningfulCharacters min: eachString size)) asLowercase = eachPrefix ] ] ].
	self assert: mcEntireWordQueryResults = smalltalkEntireWordQueryResult! !

!MagmaCollectionTester methodsFor: 'private-querys tests' stamp: 'cmm 7/11/2006 23:24'!
verifyOrder: aMagmaCollectionReader against: aSortedCollection 
	self assert: aMagmaCollectionReader size = aSortedCollection size.
	self assert: (aMagmaCollectionReader asArray: aMagmaCollectionReader lastKnownSize) = aSortedCollection asArray! !

!MagmaCollectionTester methodsFor: 'test querys - persistent result sets' stamp: 'cmm 10/8/2006 16:02'!
verifyPersistentResultSet
	| secondReader secondMc |
	clientSession abort.
	secondReader _ clientSession root at: 'testPersistentResultSet-secondReader'.
	secondMc _ secondReader collection.
	self assert: secondReader size = 10.
	self assert: secondMc size = 10.
	self assert: secondReader first = {50}.
	self assert: secondReader last = {41}.! !

!MagmaCollectionTester methodsFor: 'test major functions' stamp: 'cmm 7/15/2006 12:22'!
verifyReader: aMagmaCollectionReader matchesElementsOf: localCollection 
	"Because MagmaCollection indexes are sensitive only to the first few characters, test assertions were failing on multiple entries that were identical within the sensitive range because they were reversed.  This method compares the two in a way that allows this slight difference in order."
	"quick test"
	| sortedHashValues localEntries largeCollection index |
	self should: [ aMagmaCollectionReader size = localCollection size ].

	"build work collections, sortedHashValues and localEntries."
	index _ aMagmaCollectionReader sortIndex.
	sortedHashValues _ (localCollection collect: [ : each | index indexHashFor: each ]) asSortedCollection.
	localEntries _ Dictionary new.
	localCollection do: 
		[ : each | 
		(localEntries 
			at: (index indexHashFor: each)
			ifAbsentPut: [ Bag new ]) add: each ].

	"now verify the reader using at:"
	1 
		to: sortedHashValues size
		do: 
			[ : x1 | 
			| indexHashFromMagmaCollection indexedObject |
			indexedObject _ aMagmaCollectionReader at: x1.
			indexHashFromMagmaCollection _ index indexHashFor: indexedObject.
			self should: [ (sortedHashValues at: x1) = indexHashFromMagmaCollection ].
			self should: 
				[ ((localEntries at: indexHashFromMagmaCollection) occurrencesOf: indexedObject) = 1 ] ].
	"verify reader"
	largeCollection _ OrderedCollection new.
	aMagmaCollectionReader do: [ : each | largeCollection add: each ].
	1 
		to: sortedHashValues size
		do: 
			[ : x2 | 
			self should: 
				[ (sortedHashValues at: x2) = (index indexHashFor: (largeCollection at: x2)) ].
			self should: 
				[ ((localEntries at: (sortedHashValues at: x2)) occurrencesOf: (largeCollection at: x2)) = 1 ] ]! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 4/15/2005 17:11'!
verifyReaderForNpIntegrationTest

	self verifyReaderForNpIntegrationTest: (clientSession root at: 'testNpIntegrationForReaders')! !

!MagmaCollectionTester methodsFor: 'test non-persistent integration' stamp: 'cmm 2/19/2008 22:11'!
verifyReaderForNpIntegrationTest: aMagmaCollectionReader
	| toBeRemoved |

"Here are what the Reader contents.  The indented are the ones added afterward and now integrated.
	1 capper
2 charlie denise
	3 a b connie
4 charlie denise
5 tom dick harry
	6 millie dora
7 edward frank
8 edward frank
9 larry greg
10 tom dick harry
11 jack karen
	12 jackie kennedy
13 jack karen
	14 jackie kennedy
15 larry greg
	16 millie dora
17 nancy ozzie
	18 zebra oolong"

	"First, add some out of the range."
	aMagmaCollectionReader collection
		add: #('abba' 'baab')->Object new ;
		add: #('yappo' 'zulu')->Object new.
	self should: [ aMagmaCollectionReader collection size = 15 ].

	"Original sizes should be intact."
	self should: [ aMagmaCollectionReader size = 11 ].
	self should: [ (aMagmaCollectionReader at: 1) key = #('charlie' 'denise') ].
	self should: [ (aMagmaCollectionReader at: 2) key = #('charlie' 'denise') ].
	self should: [ (aMagmaCollectionReader at: 10) key = #('larry' 'greg') ].
	self should: [ (aMagmaCollectionReader at: 11) key = #('nancy' 'ozzie') ].

	"Now add some within the aMagmaCollectionReaders selected range."
	aMagmaCollectionReader collection
		add: #('a' 'b' 'connie')->Object new ;  "connie is the in-range keyword here"
		add: #('millie' 'dora')->Object new ;
		add: #('jackie' 'kennedy')->Object new.

	"All three of the above should now be seamlessly merged in the aMagmaCollectionReader."
	self should: [ (aMagmaCollectionReader at: 1) key = #('charlie' 'denise') ].
	"'connie' is the qualifying key here, not a and b."
	self should: [ (aMagmaCollectionReader at: 2) key = #('a' 'b' 'connie') ].
	self should: [ (aMagmaCollectionReader at: 3) key = #('charlie' 'denise') ].
	self should: [ (toBeRemoved := aMagmaCollectionReader at: 4) key = #('tom' 'dick' 'harry') ].
	self should: [ (aMagmaCollectionReader at: 5) key = #('millie' 'dora') ].
	self should: [ (aMagmaCollectionReader at: 6) key = #('edward' 'frank') ].
	self should: [ (aMagmaCollectionReader at: 7) key = #('edward' 'frank') ].
	self should: [ (aMagmaCollectionReader at: 8) key = #('larry' 'greg') ].
	self should: [ (aMagmaCollectionReader at: 9) key = #('tom' 'dick' 'harry') ].
	self should: [ (aMagmaCollectionReader at: 10) key = #('jack' 'karen') ].
	self should: [ (aMagmaCollectionReader at: 11) key = #('jackie' 'kennedy') ].
	self should: [ (aMagmaCollectionReader at: 12) key = #('jack' 'karen') ].
	self should: [ (aMagmaCollectionReader at: 13) key = #('jackie' 'kennedy') ].
	self should: [ (aMagmaCollectionReader at: 14) key = #('larry' 'greg') ].
	self should: [ (aMagmaCollectionReader at: 15) key = #('millie' 'dora') ].
	self should: [ (aMagmaCollectionReader at: 16) key = #('nancy' 'ozzie') ].
	
	"Now add to before the first persistent and see if it can correctly interleave."
	aMagmaCollectionReader collection add: #('capper')->Object new.  "<- within the range, but before charlie."
	self should: [ aMagmaCollectionReader size = 17 ].
	self should: [ (aMagmaCollectionReader at: 1) key = #('capper') ].

	"Now add to after the last persistent and see if it can correctly interleave."
	aMagmaCollectionReader collection add: #('zebra' 'oolong')->Object new.  "<- oolong within the range, but after nancy."
	self should: [ aMagmaCollectionReader size = 18 ].
	self should: [ (aMagmaCollectionReader at: 18) key = #('zebra' 'oolong') ].

	"Now lets remove from what would be prior to our current page."
	aMagmaCollectionReader collection remove: toBeRemoved.
	self should: [ aMagmaCollectionReader collection size = 19 ].
	self should: [ aMagmaCollectionReader size = 16 ].
	self should: [ (aMagmaCollectionReader at: 3) key = #('a' 'b' 'connie') ].
	self should: [ (aMagmaCollectionReader at: 5) key = #('millie' 'dora') ].
	self should: [ (aMagmaCollectionReader at: 11) key = #('jack' 'karen') ].
	self should: [ (aMagmaCollectionReader at: 16) key = #('zebra' 'oolong') ]! !

!MagmaCollectionTester methodsFor: 'private' stamp: 'cmm 7/15/2006 12:21'!
verifyReaderMatchesLocalAt: rootKeyString on: indexAttribute 
	| reader |
	clientSession abort.	"refresh to get all changes."
	reader _ (clientSession root at: rootKeyString) where: 
		[ : rdr | 
		rdr 
			read: indexAttribute
			from: '0'
			to: 'zzzzzzzzzzzzzz' ].
	self 
		verifyReader: reader
		matchesElementsOf: (localRoot at: rootKeyString)! !

!MagmaCollectionTester methodsFor: 'test MagmaSet' stamp: 'cmm 2/2/2005 09:30'!
verifyReducedMagmaSet

	self should: [ (clientSession root at: 'testMagmaSet') isEmpty ]! !

!MagmaCollectionTester methodsFor: 'test backup/restore' stamp: 'cmm 5/7/2008 17:08'!
verifyRestore
	"A backup was performed a long time ago at the beginning of the test suite.  Since then, commit.log files have been written.  Now lets apply all of those files to the backup and verify lots of things are correct just like they are in the original."
	"Close the current commit log, so we know it we'll be caught up to the second."
	| backupSession verifier recoveryOccurred correctCommitNumber |
	recoveryOccurred := false.
	correctCommitNumber := self 
		remotePerform: #copyCommitLogsToBackup
		in: 'server'.
"Encountered situation where, even though destination files were closed, they were not yet flushed, resulting in a recovery failure.  Give it five seconds, should be more than enough time.  Really, StandardFileStream>>#close should be fixed to do a #flush first.."
(Delay forSeconds: 5) wait.
	[ backupSession := MagmaSession openLocal: self backupDirectory fullName ] 
		on: MagmaForwardRecoveryNeeded
		do: 
			[ : warning | 
			recoveryOccurred := true.
			warning resume ].
	self assert: recoveryOccurred.
	self assert: correctCommitNumber = backupSession repositoryController commitNumber.
	backupSession connectAs: 'restore verifier'.
	self 
		should: [ backupSession begin ]
		raise: MagmaUserError.
	backupSession abort.
	verifier := self class new clientSession: backupSession.
	self restoreVerifications do: [ : each | verifier perform: each ].
	backupSession repositoryController commitRestore.
	backupSession
		begin ;
		commit.
	"Ok, I'm satisfied."
	backupSession disconnectAndClose! !

!MagmaCollectionTester methodsFor: 'test backup/restore' stamp: 'cmm 6/23/2007 16:26'!
verifyRestorePreqreqsCompleted
	^ VerifyRestorePreqreqsCompleted ifNil: [ VerifyRestorePreqreqsCompleted _ Set new ]! !

!MagmaCollectionTester methodsFor: 'private-querys tests' stamp: 'cmm 7/18/2006 22:56'!
verifyWhere: mcSelectBlock from: aMagmaCollection withSelect: ocSelectBlock from: anOrderedCollection 
	| reader subcollection |
	reader _ aMagmaCollection where: mcSelectBlock.
	subcollection _ anOrderedCollection select: ocSelectBlock.
	reader isSorted 
		ifTrue: 
			[ self 
				assertMc: reader
				equals: subcollection ]
		ifFalse: 
			[ self 
				assertMc: reader
				includesAll: subcollection.
			#(#first #second ) do: 
				[ : each | 
				| sortedReader |
				sortedReader _ reader 
					sortBy: each
					makeDistinct: true.
				self 
					assertMc: sortedReader
					equals: subcollection.
				sortedReader descend.
				self 
					assertMc: sortedReader
					equals: subcollection ] ]! !

!MagmaCollectionTester methodsFor: 'private-querys tests' stamp: 'cmm 8/21/2006 12:17'!
whereAndSelectBlockPairs
	"Answer a collection of two-element Arrays.  The first-element of each Array is the where block, the second is the corresponding select block.  These are used to verify MagmaCollection(Readers) against the Smalltalk collections."
	^ { 
		(Array 
			with: 
				[ : reader | 
				(reader 
					read: #first
					from: 3999
					to: 4999) & (reader 
						read: #second
						from: '3/1/2006' asDate
						to: '4/1/2015' asDate) ]
			with: 
				[ : each | 
				(each first 
					between: 3999
					and: 4999) & (each second 
						between: '3/1/2006' asDate
						and: '4/1/2015' asDate) ]).

		"disjunction, > one-page of results"
		(Array 
			with: 
				[ : reader | 
				(reader 
					read: #first
					from: 3999
					to: 4999) | (reader 
						read: #second
						from: '3/1/2006' asDate
						to: '4/1/2015' asDate) ]
			with: 
				[ : each | 
				(each first 
					between: 3999
					and: 4999) | (each second 
						between: '3/1/2006' asDate
						and: '4/1/2015' asDate) ]).

		"0 results"
		(Array 
			with: [ : reader | reader read: #first at: 3005 ]
			with: [ : each | each = 3005 ]).
		"complex (MaExpression composition), with keywords"
		(Array 
			with: 
				[ : reader | 
				(reader 
					read: #third
					from: 'ma'
					to: 'p') | ((reader 
						read: #first
						from: 3999
						to: 4999) | (reader 
							read: #second
							from: '3/1/2006' asDate
							to: '4/1/2015' asDate)) ]
			with: 
				[ : each | 
				(each third anySatisfy: 
					[ : any | 
					"sigh, must match keyword index strippage of out-off-range characters.."
					(any asUppercase select: [ : e | e asciiValue between: 33 and: 96])
						between: 'MA'
						and: 'P' ]) | ((each first 
						between: 3999
						and: 4999) | (each second 
							between: '3/1/2006' asDate
							and: '4/1/2015' asDate)) ])
	 }! !

!MagmaTestCase class methodsFor: 'other squeak images' stamp: 'cmm 8/26/2005 15:00'!
client1ImageFilename

	^ '_magmaTestClient1'! !

!MagmaTestCase class methodsFor: 'other squeak images' stamp: 'cmm 8/26/2005 15:00'!
client2ImageFilename

	^ '_magmaTestClient2'! !

!MagmaTestCase class methodsFor: 'private' stamp: 'cmm 7/16/2007 22:10'!
createRepository: aString root: anObject 
	[ MagmaRepositoryController
		delete: aString ;
		create: aString
			root: anObject ] 
		on: MagmaDeleteWarning
		do: [ : warning | warning resume ]! !

!MagmaTestCase class methodsFor: 'private' stamp: 'cmm 7/24/2008 10:17'!
dontAcceptChangeLogging
	"This method answers the source code to compile a new method on MaObject class so that, when the 'Magma server' package is unloaded, it won't thrash the hard drive so much."
	^ 'acceptsLoggingOfCompilation
	^ false'! !

!MagmaTestCase class methodsFor: 'testing' stamp: 'cmm 8/19/2007 21:11'!
isClient1Image
	^ (FileDirectory localNameFor: SmalltalkImage current imageName) = (self client1ImageFilename , '.image')! !

!MagmaTestCase class methodsFor: 'testing' stamp: 'cmm 8/19/2007 21:11'!
isClient2Image
	^ (FileDirectory localNameFor: SmalltalkImage current imageName) = (self client2ImageFilename , '.image')! !

!MagmaTestCase class methodsFor: 'testing' stamp: 'cmm 8/19/2007 21:11'!
isServerImage
	^ (FileDirectory localNameFor: SmalltalkImage current imageName) = (self serverImageFilename , '.image')! !

!MagmaTestCase class methodsFor: 'launch test suite' stamp: 'cmm 6/23/2007 14:28'!
kickoffWithoutWarning
	MagmaPreferences debug: true.
	self ensureTestEnvironmentDirectoryNamed: 'Magma'.
	"Make sure multiple files get tested by reducing it way down."
	MaAtomicFileStream maxPhysicalFileSize: 30000.
	"Make sure we test recovery requiring it to allocate extra space for recovery records."
	MaRecoveryManager defaultRecordSize: 25.	"way too small"
	super kickoffWithoutWarning.
	self isConductorImage ifTrue: 
		[ self new createRepository.
		self maDebugSuite.
		MagmaUtilitiesTester suite maDebug ]! !

!MagmaTestCase class methodsFor: 'private' stamp: 'cmm 1/6/2003 22:30'!
loadCode

	self new
		loadOldVersionsOfAllClassesInAllClients;
		remotePerform: #removeDomainClasses in: 'server'! !

!MagmaTestCase class methodsFor: 'accessing' stamp: 'cmm 7/5/2008 14:31'!
magmaServerHost
	^ MagmaServerAddress 
		ifNil: [ MagmaServerAddress := self getHostNameFor: 'server' ]
		ifNotNil: [ MagmaServerAddress ]! !

!MagmaTestCase class methodsFor: 'accessing' stamp: 'cmm 7/5/2008 14:31'!
magmaServerLocation
	^ MagmaRemoteLocation 
		host: self magmaServerHost
		port: self defaultServerPort! !

!MagmaTestCase class methodsFor: 'accessing' stamp: 'cmm 2/9/2005 16:21'!
projectsToTest

	^ #('Building with Squeak' 'Fun with Morphic' 'Fun With Music' 'SmalltalkIntroduction')! !

!MagmaTestCase class methodsFor: 'private' stamp: 'cmm 7/24/2008 12:58'!
removeServerPackage
	Utilities setAuthorInitials: 'Just Avoiding the UI!!'.
	MaObject class compile: self dontAcceptChangeLogging.
	TimeProfileBrowser onBlock: [ (MCPackage named: 'Magma server') workingCopy unload ].
	MaObject class removeSelector: #acceptsLoggingOfCompilation! !

!MagmaTestCase class methodsFor: 'private' stamp: 'cmm 8/27/2002 20:24'!
resetServerAddress

	MagmaServerAddress _ nil! !

!MagmaTestCase class methodsFor: 'other squeak images' stamp: 'cmm 8/26/2005 14:59'!
serverImageFilename

	^ '_magmaTestServer'! !

!MagmaTestCase class methodsFor: 'suites' stamp: 'cmm 6/23/2007 00:33'!
setUpSuite
	super setUpSuite.
	InitiateBackupPrereqsCompleted _ Set new.
	VerifyRestorePreqreqsCompleted _ Set new.
	self
		resetServerAddress ;
		addPlayerNamed: 'client1' ;
		addPlayerNamed: 'client2' ;
		addPlayerNamed: 'server' ;
		loadCode.
	self new
		playersDo: [ : each | each setTestCaseClass: self ] ;
		remotePerform: #startServer
			in: 'server' ;
		connectClients! !

!MagmaTestCase class methodsFor: 'suites' stamp: 'cmm 8/19/2007 21:12'!
tearDownSuite
	self isServerImage ifTrue: [ self assert: (Smalltalk hasClassNamed: #MagmaCollectionIndex) not ].
	super tearDownSuite.
	self new
		disconnectClients ;
		remotePerform: #stopServer
			in: 'server'.
	self playersDo: 
		[ : each | 
		each 
			remotePerform: #restoreRenamedClasses
			withArguments: #() ]! !

!MagmaTestCase class methodsFor: 'private' stamp: 'cmm 8/24/2005 17:19'!
validate
	"Project saving is part of the test, ensure we have some of the juicy projects that come included with the 'full' image."

	| missingProjects |
	super validate.
	missingProjects := self projectsToTest select: [ : each | (Project named: each) isNil ].
	missingProjects ifNotEmpty:
		[ Warning signal: 'Normally, these test cases want to test committing and retrieving ', self projectsToTest printString, '.  This is why it is recommended to run the test cases in the "full" image.  However ', missingProjects printString, ' are missing and, therefore, cannot be tested.' ]! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 4/12/2005 21:27'!
abort

	clientSession ifNotNil:
		[ clientSession abort.
		Smalltalk garbageCollect.
		clientSession finalizeOids ]! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 4/12/2005 20:59'!
abortClients

	self
		remotePerform: #abort
		in: 'client1'.
	self
		remotePerform: #abort
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test grow an object' stamp: 'cmm 12/15/2006 21:37'!
addAndGrowObject

	| c arr |
	c _ Set new addAll: (101 to: 108); yourself.
	"Give it its own family, so we don't go trying to save the whole default family."
	arr _ Array
		with: c
		with: nil
		with: self peopleSample1.
	clientSession begin.
	clientSession root at: 'testGrowObject' put: arr.
	clientSession commit.

	"grow the collection, forcing its buffer to be moved in the file.  Also tests commitAndBegin."
	clientSession begin.
	c addAll: (109 to: 259).
	clientSession commitAndBegin.

	(clientSession root at: 'testGrowObject') at: 2 put: '2'.
	clientSession commit! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 9/18/2006 22:37'!
addContactMethodsAttribute

	MagmaTestObject 
		subclass: #MagmaContact
		instanceVariableNames: 'contactMethods name address unused'
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Magma tester-Test domains'.! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 8/14/2007 22:25'!
addIndexAndVerify
	| mc |
	self assert: original34990 key = 34990.
	mc _ clientSession root at: 'BuildIndexCommitConflictTest_mc'.
	mc addIndex: (MaIntegerIndex attribute: #key).
	clientSession commit.
	"addIndex caused many transaction-boundary crossings."
	self assert: original34990 key = 34991.
	self assert: (mc where: [ : each | each key equals: 34990 ]) size = 0.
	self assert: (mc where: [ : each | each key equals: 34991 ]) size = 1! !

!MagmaTestCase methodsFor: 'test commit object knot' stamp: 'cmm 10/22/2004 15:55'!
addKnot

	| root |
	root _ clientSession root.
	clientSession commit:
		[ | knot |
		knot _ root
			at: 'a knotty object'
			put: MaFixtureFactory new knot.
		root
			at: 'knotty object sizes'
			put: (knot collect: [ : e | e size ]) ].
	^nil! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 6/22/2007 11:17'!
addLocalLocations
	clientSession abort.
	localRoot 
		at: 'locations'
		put: (clientSession root at: 'locations')! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 6/22/2007 11:19'!
addLocalPeople
	clientSession abort.
	localRoot 
		at: 'people'
		put: (clientSession root at: 'people')! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 6/22/2007 11:17'!
addLocations
	"This test is done as an identity test.  Each 'person' has an address.  If we reference an address from two places, we want to change it in place#1 on client1 and then verify it is changed in both place#1 and place#2 in client2."
	self 
		remotePerform: #addPersistentLocations
		in: 'client1'.
	self 
		remotePerform: #addLocalLocations
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 6/22/2007 11:19'!
addPeople
	self 
		remotePerform: #addPersistentPeople
		in: 'client1'.
	"make sure localRoot has it in client2."
	self 
		remotePerform: #addLocalPeople
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 6/22/2007 11:16'!
addPersistentLocations
	| people root |
	root _ clientSession root.
	people _ self clientSession root at: 'people'.
	self clientSession commit: 
		[ root 
			at: 'locations'
			put: (people collect: [ : each | each address ]) ]! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 6/22/2007 11:18'!
addPersistentPeople
	| root |
	root _ clientSession root.
	self clientSession commit: 
		[ root 
			at: 'people'
			put: self peopleSample1 ]! !

!MagmaTestCase methodsFor: 'test objects' stamp: 'cmm 4/8/2005 15:35'!
addSamples

	| samples |
	clientSession preferences refreshPersistentObjectsEvenWhenChangedOnlyByMe: true.
	samples _ MaFixtureFactory new samples.
	clientSession commit:
		[ clientSession root
			at: 'samples'
			put: samples ].
	"Test that all kinds of the samples can withstand the potential refresh or not."
	clientSession abort.
	clientSession preferences refreshPersistentObjectsEvenWhenChangedOnlyByMe: false! !

!MagmaTestCase methodsFor: 'test blocks' stamp: 'cmm 4/7/2005 15:48'!
addSortBlock

	clientSession commit: [ clientSession root at: 'testBlocks' put: self sortBlock ].
	"to be sure it survives a refresh-attempt"
	clientSession commit: [ ]! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 6/6/2005 23:11'!
addToMcFrom: aClass at: aString

	| mc |
	mc _ clientSession root at: aString.
	aClass withAllSubclassesDo: [ : each | clientSession commit: [ mc add: each maCommentKeywords -> each name ] ]! !

!MagmaTestCase methodsFor: 'test advance' stamp: 'cmm 8/13/2008 00:13'!
advanceAndVerify
	clientSession advance.
	self assert: objectToAdvance first = 'object changed'! !

!MagmaTestCase methodsFor: 'test forwarding proxys' stamp: 'cmm 4/10/2005 18:00'!
agitateRemoteProxy
	"Agitate my forwarding proxy in A.  Since B is offline, verify offline-object created."

	| didReceiveNotification |
	proxy := clientSession root at: 'testAutoConnectingForwardingProxy'.
	self should: [proxy maIsForwardingProxy].
	didReceiveNotification := false.
	self should: 
			[[proxy value maOriginalClass = MagmaOfflineObject] 
				on: MagmaUserRequiredNotification
				do: 
					[:request | 
					didReceiveNotification := true.
					request resume: (MagmaUser id: 'client2')]].
	self should: [didReceiveNotification].
	self should: [proxy hasOfflineResults]! !

!MagmaTestCase methodsFor: 'test forwarding proxys' stamp: 'cmm 4/18/2005 16:44'!
agitateRemoteProxyAgain
	"Agitate my forwarding proxy in A again.  Since B is now back online, therefore the offline object should become an online object."

	| copyOfRemote didReceiveWarning |
	self should: 
			[(clientSession root at: 'testAutoConnectingForwardingProxy') 
				maIsForwardingProxy].
	self should: [proxy value maOriginalClass = MagmaOfflineObject].
	proxy proxyReconnect.
	copyOfRemote := self remoteObject.
	didReceiveWarning := false.
	self should: 
			[[proxy value = copyOfRemote value] on: MagmaWarning
				do: 
					[:warning | 
					didReceiveWarning := true.
					warning resume]].
	self assert: didReceiveWarning.
	self should: [proxy key = copyOfRemote key].
	self shouldnt: [proxy hasOfflineResults].
	"All done, disconnect."
	proxy proxyDisconnect! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 6/13/2007 22:59'!
assertForwardRecoverySignaled
	self assert: forwardRecoverySignaled = true! !

!MagmaTestCase methodsFor: 'test backup/restore' stamp: 'cmm 6/22/2007 11:04'!
backupDirectory
	^ FileDirectory on: (self environmentDirectory fullNameFor: 'MagmaBackup')! !

!MagmaTestCase methodsFor: 'testBecome' stamp: 'cmm 12/7/2004 14:42'!
become

	clientSession commit:
		[ | array |
		(clientSession root at: #testBecomeToTransient) become: { 'new'. 'becomed'. 'array' }.
		array _ (clientSession root at: #testBecomeToPersistent).
		array first become: array last ]! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/22/2007 12:21'!
bifMuscleIn
	| paula bif |
	clientSession begin.
	paula _ (clientSession root at: 'people') first.
	bif _ (clientSession root at: 'people') last.
	bif marry: paula.
	localRoot 
		at: 'performCommitConflictTest'
		put: bif	"avoid gc"! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 6/8/2005 21:39'!
buildExistingObjectsForCommitDisaster

	clientSession commit:
		[ clientSession root
			at: 'existing-objects for commitDisaster'
			put:
				((500 to: 600) collect: [ : e | e printString -> Object new ]).
		clientSession root
			at: 'object to grow for commitDisaster'
			put: ((OrderedCollection new: 2) add: 1; add: 2; yourself) ]! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 6/7/2005 13:11'!
buildIndexedMagmaCollectionFrom: aClass at: aString

	| indexDef mc |
	mc _ MagmaCollection new.
	indexDef _ self newKeywordIndex.
	mc addIndex: indexDef.
	clientSession commit: [ clientSession root at: aString put: mc ].
	self
		addToMcFrom: aClass
		at: aString! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 6/6/2005 23:05'!
buildMagmaArray

	| ma |
	ma _ MagmaArray new.
	1 to: 100 do: [ : e | ma at: e put: e printString ].
	clientSession commit:
		[ clientSession root
			at: 'existing-MagmaArray for commitDisaster'
			put: ma ]! !

!MagmaTestCase methodsFor: 'test refreshAll' stamp: 'cmm 2/11/2008 22:14'!
cacheDormantAndDisconnect
	dormantObject := clientSession root at: #testRefreshAll.
	clientSession disconnect! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 9/12/2007 21:52'!
cacheFlushFrequency: numberOfSeconds 
	magmaConsole controller cacheFlushFrequency: numberOfSeconds! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 4/28/2007 15:57'!
captureFileChecksumsWithCopy: aBoolean 
	| answer |
	magmaConsole shutdown.
	(aBoolean and: [ MagmaServerPreferences debugRecovery ]) ifTrue: [ self copyDb ].
	answer _ self fileChecksums.
	magmaConsole open: self environmentPath.
	magmaConsole processOn: self class defaultServerPort.
	^ answer! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 10/26/2004 23:50'!
changeName

	"This should warn us we're about to truncate data because this clients MagmaPerson doesn't have address."
	2 timesRepeat:
		[ self should:
			[
			[ clientSession commit: [ (localRoot at: 'testClassSchemaUpgrades') name: 'Muller' ].
			false ]
				on: MagmaTruncationWarning
				do:
					[ : warning | clientSession abort.  (warning messageText findString: 'MagmaPerson') > 0 ] ] ].

	"Now load the latest code committed by another client so we don't get that warning."
	clientSession definition codeBase installClasses.
	"gotta stub it out so it re-reads it, otherwise warning will continue.."
	clientSession stubOut: (localRoot at: 'testClassSchemaUpgrades').
	clientSession commit: [ (localRoot at: 'testClassSchemaUpgrades') name: 'Muller' ].
	self maMarked: 'magic'.  "calling contactMethods works due to DNU override in MagmaTestObject"
	self should: [ (localRoot at: 'testClassSchemaUpgrades') contactMethods = (Dictionary new at: #address put: '101 W. Meyer'; yourself) ].! !

!MagmaTestCase methodsFor: 'test advance' stamp: 'cmm 8/13/2008 00:12'!
changeObjectToAdvance
	clientSession commit: [ (clientSession root at: #testAdvance) at: 1 put: 'object changed' ]! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/22/2007 12:03'!
changePerson1
	| person |
	clientSession begin.
	person _ (clientSession root at: 'people') first.
	person name first: 'elizabeth'.
	localRoot 
		at: 'performCommitAndRefreshTest'
		put: person! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/22/2007 12:19'!
chrisMarryPaula
	| paula chris |
	clientSession begin.
	paula _ (clientSession root at: 'people') first.
	chris _ (clientSession root at: 'people') at: 2.
	chris marry: paula.
	localRoot 
		at: 'performCommitConflictTest'
		put: chris	"avoid gc"! !

!MagmaTestCase methodsFor: 'client' stamp: 'cmm 5/23/2002 21:12'!
clientSession
	"accessible from client1 and client2"

	^clientSession
! !

!MagmaTestCase methodsFor: 'test backup/restore' stamp: 'cmm 6/21/2007 21:22'!
clientSession: aMagmaSession
	clientSession _ aMagmaSession! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 1/15/2005 13:50'!
close

	clientSession
		ifNotNil: 
			[ self clientSession disconnect.
			self clientSession closeRepository ].
	magmaConsole ifNotNil: [ self stopServer ].
	peerConsole ifNotNil: [ self stopPeer ]! !

!MagmaTestCase methodsFor: 'client' stamp: 'cmm 5/7/2002 22:51'!
commit

	clientSession commit.
	^nil
! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 5/25/2007 00:04'!
commitDisaster
	"simulate an outage at the worst-possible time; after a commit which includes added and removed keys of an existing MagmaCollection (with index) and MagmaArray, some, new objects, a new class-def so it changes that part of its definition too, and a new MagmaArray (to test it removes files).  Just after has written the applied the commit to all the various files, it then exits the image without saving and without having written the end-apply tag record.  This will exercise all file types."
	| existingCollection reader newStuff existingArray |
	clientSession begin.
	existingCollection _ clientSession root at: 'existing-collection for commitDisaster'.
	reader _ existingCollection where: 
		[ : rdr | 
		rdr 
			read: #key
			from: 'x'
			to: 'z' ].
	self should: [ reader size > 1	"let's ensure at least a couple of removes to test" ].
	"remove a bunch.."
	reader asOrderedCollection do: [ : each | existingCollection remove: each ].
	"now add a bunch of new stuff."
	Number withAllSubclassesDo: 
		[ : each | 
		existingCollection add: each maCommentKeywords -> each name ].
	"Update the MagmaArray, extending its length.."
	existingArray _ clientSession root at: 'existing-MagmaArray for commitDisaster'.
	50 
		to: 150
		do: 
			[ : each | 
			existingArray 
				at: each
				put: '#' , each printString ].
	"Create some new objects."
	newStuff _ OrderedCollection new.
	newStuff
		add: MagmaArray new ;
		add: ((1 to: 100) collect: 
				[ : each | 
				"at the time this test runs, this should be a new class too."
				Object new ]).
	"Update some old objects."
	(clientSession root at: 'existing-objects for commitDisaster') do: 
		[ : eachAssociation | 
		| temp |
		temp _ eachAssociation key.
		eachAssociation key: eachAssociation value.
		eachAssociation value: temp ].
	(clientSession root at: 'object to grow for commitDisaster') addAll: (3 to: 10).
	"Now do the fatal commit.."
	clientSession root 
		at: 'this key should NOT exist!!'
		put: newStuff.
	clientSession commit! !

!MagmaTestCase methodsFor: 'test forwarding proxys' stamp: 'cmm 7/5/2008 14:25'!
commitForwardingProxy
	| sessionB |
	sessionB := MagmaSession 
		host: (self getHostNameFor: 'peer')
		port: self peerPort.
	sessionB connect: (MagmaUser id: clientSession userId).
	clientSession commit: 
		[ clientSession root 
			at: 'testAutoConnectingForwardingProxy'
			put: sessionB root asMagmaForwardingProxy ].
	sessionB disconnect! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 7/5/2008 14:31'!
commitInitialMagmaPerson
	| session |
	session := MagmaSession 
		host: self class magmaServerHost
		port: self class defaultServerPort.
	session connect: (MagmaUser id: 'conductor').
	session commit: 
		[ session root 
			at: 'testClassSchemaUpgrades'
			put: (MagmaPerson new
					name: 'Mayer' ;
					yourself) ].
	session disconnect! !

!MagmaTestCase methodsFor: 'test projects' stamp: 'cmm 5/15/2005 18:40'!
commitProjects

	| projects |
	clientSession commit:
		[ clientSession root at: 'testProjects' put: (projects _ OrderedCollection new) ].
	self class projectsToTest do:
		[ : each |  | project |
		project _ Project named: each.
		project
			ifNil: [ MagmaNotification signal: each, ' not found, nothing to commit.' ]
			ifNotNil: [ clientSession commit: [ projects add: each->project ] ] ]! !

!MagmaTestCase methodsFor: 'test storing a session' stamp: 'cmm 1/15/2007 18:38'!
commitReferenceToSession
	clientSession commit:
		[ clientSession root
			at: 'testStoringMagmaSession' 
			put: clientSession ]! !

!MagmaTestCase methodsFor: 'client' stamp: 'cmm 5/24/2005 22:53'!
connect: aMagmaLocation as: aString

	clientSession _ aMagmaLocation newSession.
	clientSession connect: (MagmaUser id: aString).
	MagmaServerAddress _ aMagmaLocation hostAddress! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 1/15/2005 16:11'!
connectClients

	self
		remotePerform: #connect:as:
		withArguments: { self class magmaServerLocation. 'client1' }
		in: 'client1'.
	self
		remotePerform: #connect:as:
		withArguments: { self class magmaServerLocation. 'client2' }
		in: 'client2'
! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 7/16/2007 22:09'!
copyDb
	"This method is used only to assist with debugging problems with recovery."
	| location targetDir |
	location _ magmaConsole controller localLocation.
	targetDir _ FileDirectory on: (location directory containingDirectory fullNameFor: location directory localName , 'Copy').
	targetDir assureExistence.
	[ MagmaRepositoryController delete: targetDir pathName ] 
		on: MagmaDeleteWarning
		do: [ : warning | warning resume ].
	location fileNames do: 
		[ : each | 
		location directory 
			maCopyFileNamed: each
			toDirectory: targetDir ]! !

!MagmaTestCase methodsFor: 'test MagmaCounter' stamp: 'cmm 7/21/2007 15:29'!
counterFixture
	^ clientSession root at: 'testMagmaCounter'! !

!MagmaTestCase methodsFor: 'test MagmaCounter' stamp: 'cmm 7/20/2007 12:00'!
createCounter
	clientSession commit: 
		[ clientSession root 
			at: 'testMagmaCounter'
			put: 
				(MagmaCounter new
					increment: 10 ;
					yourself) ]! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/21/2007 00:07'!
createMagmaCollectionAndIndexSimultaneously
	| root mc |
	root _ clientSession root.
	mc _ MagmaCollection new.
	mc addIndex: (MaDateIndex attribute: #date).
	clientSession commit: 
		[ root 
			at: 'testCreateMagmaCollectionAndIndexSimultaneously'
			put: mc ]! !

!MagmaTestCase methodsFor: 'test refreshAll' stamp: 'cmm 2/11/2008 22:12'!
createObjectToBeDormantInImage
	clientSession commit: 
		[ clientSession root 
			at: #testRefreshAll
			put: (Array with: 'original object') ]! !

!MagmaTestCase methodsFor: 'peer' stamp: 'cmm 3/11/2005 15:22'!
createPeer

	self class
		createRepository: self peerPath
		root: self remoteObject! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 6/23/2007 15:16'!
createRepository
	self class 
		createRepository: self environmentPath
		root: Dictionary new! !

!MagmaTestCase methodsFor: 'test sequence-number generator' stamp: 'cmm 9/6/2007 21:46'!
createSequenceNumberGenerator
	clientSession serverPerform: #initializeSequenceNumberGenerator! !

!MagmaTestCase methodsFor: 'test Set' stamp: 'cmm 11/30/2004 21:21'!
createTestSet

	clientSession commit:
		[ clientSession root
			at: 'testSet'
			put: (Set with: (StringHolder new contents: 'proxied string'; yourself)) ]! !

!MagmaTestCase methodsFor: 'client' stamp: 'cmm 6/18/2002 22:06'!
disconnect

	clientSession ifNotNil: [ clientSession disconnect ].
	^nil! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 12/18/2002 22:40'!
disconnectClients

	self
		remotePerform: #disconnect
		in: 'client1'.
	self
		remotePerform: #disconnect
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test projects' stamp: 'cmm 6/29/2007 23:14'!
enterProjects
	"just navigating to the projects will force their materialization and registry with Project>>'AllProjects'"
	(clientSession abort root at: 'testProjects') do: [ : eachAssociation | eachAssociation value name ].
	WorldState addDeferredUIMessage: 
		[ (Workspace new contents: (String streamContents: 
				[ : stream | 
				stream nextPutAll: 'Go into the following projects and see if they look ok.'.
				self class projectsToTest do: 
					[ : each | 
					stream
						cr ;
						cr ;
						tab ;
						nextPutAll: each.
					(Project named: each) ifNil: 
						[ stream
							space ;
							nextPutAll: 'NOT FOUND!!  Test failed.' ] ] ])) openLabel: 'Verify these Projects' ]
	"  This doesn't work even with stock projects in the image, not sure why.  Commenting out for now.."
	"	| returnTo |
	returnTo _ Project current.
	self class projectsToTest do: [ : each | (clientSession root at: each) enter ].
	returnTo enter"! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 9/26/2004 19:36'!
environmentPath

	^ (FileDirectory on: super environmentPath) fullNameFor: 'Magma'! !

!MagmaTestCase methodsFor: 'test projects' stamp: 'cmm 5/15/2005 18:35'!
expungeProjects

	self class projectsToTest do:
		[ : each |  | eachProject |
		eachProject _ Project named: each.
		eachProject
			ifNil: [ MagmaNotification signal: each, ' not found and, therefore, not tested.' ]
			ifNotNil:
				[ MagmaNotification signal: 'expunging ', each.
				ProjectHistory forget: eachProject.
				Project deletingProject: eachProject ] ]! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 6/13/2007 21:16'!
fileChecksums
	| answer location sha |
	answer _ Dictionary new.
	sha _ SecureHashAlgorithm new.
	location _ magmaConsole controller localLocation.
	location fileNames do: 
		[ : eachFilename | 
		| file |
		({  '.images'.  MaRecoveryManager commitLogExtension } anySatisfy: [ : eachExtension | eachFilename endsWith: eachExtension ]) ifFalse: 
			[ file _ (location directory readOnlyFileNamed: eachFilename) binary.
			answer 
				at: eachFilename
				put: (sha hashStream: file).
			file close ] ].
	^ answer! !

!MagmaTestCase methodsFor: 'test MagmaCounter' stamp: 'cmm 7/23/2007 14:48'!
finalValidateCounter
	| counter |
	(counter _ self counterFixture) increment: 5.
	self assert: counter value = 50.  "shared 0, delta 5"
	clientSession commit.
	"but client1 had decremented by 15, now we can see that."
	self assert: counter value = 35! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 7/8/2005 10:40'!
flushAggressively: aBoolean

	magmaConsole controller flushAggressively: aBoolean! !

!MagmaTestCase methodsFor: 'test backup/restore' stamp: 'cmm 8/3/2007 11:00'!
fullBackup
	[ magmaConsole controller fullBackupTo: self backupDirectory ] 
		on: MagmaDeleteWarning
		do: [ : warning | warning resume ]! !

!MagmaTestCase methodsFor: 'test sequence-number generator' stamp: 'cmm 9/6/2007 21:45'!
getNextSequenceNumber
	^ clientSession serverPerform: #getNextSequenceNumber! !

!MagmaTestCase methodsFor: 'test sequence-number generator' stamp: 'cmm 8/23/2007 13:07'!
getSequenceNumbers
	^ sequenceNumbers ! !

!MagmaTestCase methodsFor: 'test sequence-number generator' stamp: 'cmm 8/23/2007 13:08'!
getServerCounterValue
	clientSession abort.
	^ (clientSession root at: #testSequenceNumberGenerator) value! !

!MagmaTestCase methodsFor: 'test advance' stamp: 'cmm 8/13/2008 00:10'!
initTestAdvance
	clientSession commit: 
		[ clientSession root 
			at: #testAdvance
			put: (Array with: String new) ]! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 6/24/2007 23:37'!
initialize

	super initialize.
	localRoot _ Dictionary new.
	timeToVerifyRestore _ false! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 8/14/2007 22:20'!
initializeBuildIndexCommitConflictTest
	| objects |
	objects _ (100 
		to: 35000
		by: 5) collect: [ : n | n -> n asWords ].
	clientSession commit: 
		[ clientSession root
			at: 'BuildIndexCommitConflictTest_objects'
				put: objects ;
			at: 'BuildIndexCommitConflictTest_mc'
				put: 
					(MagmaCollection new
						addAll: objects ;
						yourself) ].
	clientSession begin.
	"Prevent garbage collection."
	original34990 _ objects at: 6979! !

!MagmaTestCase methodsFor: 'test backup/restore' stamp: 'cmm 6/23/2007 16:29'!
initiateBackupIfTime
	InitiateBackupPrereqsCompleted = self requiredPrereqsToInitiateBackup ifTrue: 
		[ self
			remotePerform: #fullBackup
			in: 'server'.
		"Prevent any other backups from being initiated."
		InitiateBackupPrereqsCompleted add: 'backup initiated' ]! !

!MagmaTestCase methodsFor: 'test after-images recovery' stamp: 'cmm 7/16/2005 18:13'!
killImage

	SmalltalkImage current quitPrimitive! !

!MagmaTestCase methodsFor: 'loading code' stamp: 'cmm 10/7/2004 16:57'!
loadChangeSet: fileName

	| oldChanges file |
	oldChanges _ ChangeSet current.
	ChangeSet newChanges: (ChangeSorter assuredChangeSetNamed: 'MagmaTestClasses').
	file _ self resourcesDirectory readOnlyFileNamed: fileName.
	[ file 
		maFileInAnnouncing: 'Loading ', file localName
		logged: false ] 

		ensure:
			[ file close.
			ChangeSet newChanges: oldChanges ]! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 10/6/2004 21:09'!
loadOldVersionsOfAllClassesInAllClients

	self
		loadChangeSet: 'MagmaContact.1.st' ;
		pushClass: (Smalltalk classNamed: #MagmaContact) ;
		loadChangeSet: 'MagmaPerson.1.st' ;
		pushClass: (Smalltalk classNamed: #MagmaPerson)! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/22/2007 12:15'!
marryPerson2
	| person locations |
	locations _ clientSession root at: 'locations'.
	self should: 
		[ (clientSession root at: 'people') allSatisfy: [ : each | locations includes: each address ] ].
	clientSession begin.
	person _ (clientSession root at: 'people') at: 3.
	person name first: 'smitty'.
	person marry: ((clientSession root at: 'people') at: 2).
	clientSession commit! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 8/17/2005 19:10'!
newKeywordIndex

	^ MaKeywordIndex attribute: #key  "key of an Association"! !

!MagmaTestCase methodsFor: 'peer' stamp: 'cmm 1/15/2005 13:46'!
peerPath

	^ (FileDirectory on: super environmentPath) fullNameFor: 'Peer'! !

!MagmaTestCase methodsFor: 'peer' stamp: 'cmm 1/15/2005 13:46'!
peerPort

	^ 51069! !

!MagmaTestCase methodsFor: 'test objects' stamp: 'cmm 10/22/2004 15:55'!
peopleSample1

	^MaFixtureFactory current newPersons: 3
! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/14/2007 22:22'!
performBuildIndexCommitConflictTest
	self
		remotePerform: #initializeBuildIndexCommitConflictTest
			in: 'client1' ;
		remotePerform: #update34990
			in: 'client2' ;
		remotePerform: #addIndexAndVerify
			in: 'client1'! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/22/2007 12:18'!
performCommitAndRefreshTest
	"client1 begins a transaction in, change person 1 name to elizabeth."
	self 
		remotePerform: #changePerson1
		in: 'client1'.
	"client2, commits person 3 name to smitty and also to marry person 2."
	self 
		remotePerform: #marryPerson2
		in: 'client2'.
	"client1 commit changes to person1, and should then see all changes by client2."
	self 
		remotePerform: #verifyCommitAlsoRefreshes
		in: 'client1'.
	"client2 should not see client1's changes until it aborts."
	self 
		remotePerform: #verifyAbortRefreshes
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/22/2007 12:23'!
performCommitConflictTest
	"client1 begins a transaction, chris marries paula."
	self 
		remotePerform: #chrisMarryPaula
		in: 'client1'.
	"client2, begins a transaction, bif tries to muscle in on paula, but he didn't make a commitment.."
	self 
		remotePerform: #bifMuscleIn
		in: 'client2'.
	"chris does.."
	self 
		remotePerform: #commit
		in: 'client1'.
	"bif will now get a commit failure, after which it should be able to see paula's spouse changes."
	self 
		remotePerform: #verifyCommitFailure
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test connectionless requests' stamp: 'cmm 7/15/2008 17:12'!
performDisconnectReconnect
	"Just disconnecting and reconnecting would force the magmaId to be validated."
	self assert: clientSession isConnected.
	clientSession
		disconnect ;
		connectAs: clientSession userId.
	self assert: clientSession magmaId class = MagmaId.
	self assert: clientSession isConnected! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/21/2007 00:25'!
performDoubleUpdate
	| root |
	root _ clientSession root.
	clientSession commit: 
		[ root 
			at: 'testDoubleUpdate'
			put: 3 -> 4 ].
	clientSession commit: 
		[ root 
			at: 'testDoubleUpdate'
			put: 'three' -> 'four' ].
	clientSession commit: 
		[ root 
			at: 'testDoubleUpdate'
			put: 5 -> 6 ]! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/21/2007 00:27'!
performSetUpStaleProxyAutoReconnect
	clientSession abort commit: 
		[ clientSession root 
			at: 'testStaleProxyAutoReconnect'
			put: (MagmaPerson new
					name: 'Benny' ;
					address: '10 Volcanoes Drive' ;
					yourself) ]! !

!MagmaTestCase methodsFor: 'private' stamp: 'cmm 6/24/2007 23:19'!
performTest
	super performTest.
	self
		initiateBackupIfTime ;
		verifyRestoreIfTime! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/22/2007 12:02'!
performUpdateByteObjects
	| root |
	root _ clientSession root.
	clientSession
		abort ;
		commit: 
			[ (root at: 'testUpdateByteObjects') first 
				at: 1
				put: $H.
			(root at: 'testUpdateByteObjects') last 
				at: 1
				put: $H asciiValue ]! !

!MagmaTestCase methodsFor: 'private' stamp: 'cmm 7/5/2008 14:52'!
playerProtocol

	^ super playerProtocol
		addAll:
			(
			#( ReadWriteStream.  "for fileouts"
			LargePositiveInteger.  "for returning an oid"
			MagmaRemoteLocation. MaServerLocation. MagmaPerson.  MagmaPersonName.  MagmaMailingAddress ) 
				select: [ : each  | Smalltalk hasClassNamed: each ] 
				thenCollect: [ : each | Smalltalk classNamed: each ]) ;
		addAll: self class withAllSubclasses ;
		yourself! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/21/2007 00:30'!
prepareUpdateByteObjects
	| root |
	root _ clientSession root.
	clientSession commit: 
		[ root 
			at: 'testUpdateByteObjects'
			put: (Array 
					with: 'hello world'
					with: 'hello world' asByteArray) ]! !

!MagmaTestCase methodsFor: 'loading code' stamp: 'cmm 10/7/2004 17:29'!
projectDirectory

	^ (self projectsDirectory directoryExists: 'Magma')
		ifTrue:
			[ FileDirectory on: (self projectsDirectory fullNameFor: 'Magma') ]
		ifFalse:
			[ self projectsDirectory ]! !

!MagmaTestCase methodsFor: 'loading code' stamp: 'cmm 5/1/2003 17:12'!
pushClass: aClass

	self playersDo: [ :each | each name ~= 'server' ifTrue: [ each pushClass: aClass ] ]! !

!MagmaTestCase methodsFor: 'test grow an object' stamp: 'cmm 12/15/2006 21:53'!
readAndVerifyGrownObject
	| arr |
	arr _ clientSession abort root at: 'testGrowObject'.
	self should: [ arr first = (101 to: 259) asSet ].
	self should: [ (arr at: 2) = '2' ].
	self should: [ arr last maEquivalentForSerializationTest: self peopleSample1 ]! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 6/6/2004 13:48'!
readPerson

	localRoot
		at: 'testClassSchemaUpgrades'
		put: (clientSession root at: 'testClassSchemaUpgrades')! !

!MagmaTestCase methodsFor: 'test refreshAll' stamp: 'cmm 2/11/2008 22:19'!
reconnectAndVerifyDormantUpdated
	clientSession connectAs: 'client2'.
	self should: dormantObject = (Array with: 'the updated value')! !

!MagmaTestCase methodsFor: 'test advance' stamp: 'cmm 8/13/2008 00:11'!
referenceObjectToAdvance
	objectToAdvance := (clientSession root at: #testAdvance) ! !

!MagmaTestCase methodsFor: 'peer' stamp: 'cmm 1/15/2005 15:39'!
remoteObject

	^ Object -> (Set withAll: 'a collection of strings' substrings)! !

!MagmaTestCase methodsFor: 'test after-images recovery' stamp: 'cmm 7/15/2005 17:20'!
remoteStringCollection

	^ (clientSession root at: 'testAfterImagesRecovery') at: clientSession userId last digitValue! !

!MagmaTestCase methodsFor: 'loading code' stamp: 'cmm 6/7/2004 22:20'!
removeDomainClasses

	#(MagmaPerson MagmaPersonName) do: [ :each | (Smalltalk removeClassNamed: each) ]! !

!MagmaTestCase methodsFor: 'test backup/restore' stamp: 'cmm 6/22/2007 12:45'!
requiredPrereqsToInitiateBackup
	"To do the backup/restore test, we commence the backup only after all of my server-killing-tests are complete because otherwise the backup could be aborted in the middle."
	^ Set 
		with: #testRollbackRecovery
		with: #testAfterImagesRecovery! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 6/29/2004 23:43'!
restoreRenamedClasses

	(Smalltalk hasClassNamed: #MagmaRenamedContact) ifTrue:
		[ (Smalltalk at: #MagmaRenamedContact) rename: #MagmaContact ]! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 1/18/2007 23:47'!
runLocalSessionsConcurrencyDetection
	| session1 session2 location |
	location := MagmaLocalLocation path: self environmentPath.
	"Quick test of MagmaLocalLocation #=, get the controller by looking it up that way."
	session1 := MagmaSession repositoryController: (MagmaRepositoryController controllerOpenedOn: location).
	session1 connectAs: 'session1'.
	session2 := MagmaSession repositoryController: magmaConsole controller.
	session2 connectAs: 'session2'.
	[ session1 begin.
	session1 root at: #foo put: #bar.
	session2 commit: [session2 root at: #foo put: #baz].
	self
		should: [ session1 commit ]
		raise: MagmaCommitError ]

		ensure: 
			[ session1 disconnect.
			session2 disconnect ]! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 1/25/2005 15:37'!
setContactMethodsAndCode

	clientSession commit: 
		[ self maMarked: 'magic'.  "setting contactMethods: works due to DNU override in MagmaTestObject"
		(clientSession root at: 'testClassSchemaUpgrades') contactMethods: (Dictionary new at: #address put: '101 W. Meyer'; yourself).
		clientSession definition codeBase fileOutClass: (Smalltalk at: #MagmaContact).
		"To make this even more intense, let's rename MagmaContact to MagmaRenamedContact!!"
		(Smalltalk at: #MagmaContact) rename: #MagmaRenamedContact ]! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 6/6/2005 21:59'!
setSimulateServerOutage: aBoolean

	MaObjectRepository simulateOutage: true! !

!MagmaTestCase methodsFor: 'private' stamp: 'cmm 4/12/2005 20:59'!
setUp
	"Ensure we have a fresh view for each test."

	self abortClients! !

!MagmaTestCase methodsFor: 'test after-images recovery' stamp: 'cmm 8/12/2007 14:51'!
setUpAfterImagesRecovery
	clientSession link timeoutSeconds: 4.
	clientSession commit:
		[ clientSession root 
			at: 'testAfterImagesRecovery' 
			put: { OrderedCollection new: 1. OrderedCollection new: 1 } ]! !

!MagmaTestCase methodsFor: 'testBecome' stamp: 'cmm 12/7/2004 21:01'!
setUpTestBecome
	clientSession commit:
		[ { localRoot.  clientSession root } do:
			[ : each |
			each at: #testBecomeToTransient put: { 'non becomed array' }.
			each at: #testBecomeToPersistent put: { 'becomer'.  'becomee with different size' } ] ]! !

!MagmaTestCase methodsFor: 'test blocks' stamp: 'cmm 11/12/2004 17:06'!
sortBlock

	^ [ : a : b | a name reverse asUppercase > b name reverse asUppercase ]! !

!MagmaTestCase methodsFor: 'test after-images recovery' stamp: 'cmm 5/2/2007 22:05'!
startAddStrings
	| oc serverDown counter |
	oc _ self remoteStringCollection.
	serverDown _ false.
	afterImagesProgress _ OrderedCollection new.
	counter _ 1.
	clientSession begin.
	[ [ serverDown ] whileFalse:
		[ oc add: counter printString.
		[ clientSession commitAndBegin.
		"When the server is ripped down, the last commit, above, either succeeded or it didn't, so we must be forgiving by one when we verify."
		afterImagesProgress add: counter printString.
		counter _ counter + 1 ]
			on: MaNetworkError, MagmaSessionLost 
			do: [ : err | serverDown _ true ] ] ] fork! !

!MagmaTestCase methodsFor: 'test sequence-number generator' stamp: 'cmm 8/23/2007 13:04'!
startGeneratingSequenceNumbers
	shouldStopGeneratingSequenceNumbers _ false.
	sequenceNumbers _ OrderedCollection new.
	[ [ shouldStopGeneratingSequenceNumbers ] whileFalse: [ sequenceNumbers add: self getNextSequenceNumber ].
	shouldStopGeneratingSequenceNumbers _ 'done' ] fork name: 'generating sequence numbers'! !

!MagmaTestCase methodsFor: 'peer' stamp: 'cmm 7/18/2005 13:21'!
startPeer

	peerConsole _
		MagmaServerConsole new 
			open: self peerPath ;
			processOn: self peerPort ;
			yourself! !

!MagmaTestCase methodsFor: 'initializing' stamp: 'cmm 5/9/2008 16:40'!
startServer
	[ magmaConsole := MagmaServerConsole new
		open: self environmentPath ;
		processOn: self class defaultServerPort ;
		yourself.
	magmaConsole statistics snapshotInterval: (1 to: 30 by: 1) ] 
		on: MagmaForwardRecoveryNeeded
		do: 
			[ : warning | 
			forwardRecoverySignaled := true.
			warning resume ]! !

!MagmaTestCase methodsFor: 'test sequence-number generator' stamp: 'cmm 8/23/2007 12:59'!
stopGeneratingSequenceNumbers
	shouldStopGeneratingSequenceNumbers _ true! !

!MagmaTestCase methodsFor: 'peer' stamp: 'cmm 1/15/2005 13:49'!
stopPeer

	peerConsole shutdown! !

!MagmaTestCase methodsFor: 'peer' stamp: 'cmm 12/18/2002 21:22'!
stopServer

	magmaConsole shutdown! !

!MagmaTestCase methodsFor: 'test advance' stamp: 'cmm 8/13/2008 00:05'!
testAdvance
	self
		remotePerform: #initTestAdvance
			in: 'client1' ;
		remotePerform: #referenceObjectToAdvance
			in: 'client2' ;
		remotePerform: #changeObjectToAdvance
			in: 'client1' ;
		remotePerform: #advanceAndVerify
			in: 'client2'! !

!MagmaTestCase methodsFor: 'test after-images recovery' stamp: 'cmm 6/4/2008 23:30'!
testAfterImagesRecovery
	"- kickoff background process in both clients that add strings to their own OrderedCollection.
		root
			'testDisasterRecovery' -> Array (2-elements)
				1->OC1
				2->OC2
	- each client keeps track of how far it is.
	- Delay for a few seconds while they chug.
	- kill the server, wait for shutdown.
	- client error-handling (MaNetworkError) has ended their add loop.  Last successful string is known in memory.
	- restart server image, recovery will happen automatically
	- wait for server to respond
	- disconnectClients; connectClients
	- clients verify their own OC against in-memory progress."
	self
		remotePerform: #flushAggressively:
			with: true
			in: 'server' ;
		remotePerform: #setUpAfterImagesRecovery
			in: 'client1'.
	(Delay forSeconds: 5) wait.	"for flushCache of setup"
	self 
		remotePerform: #cacheFlushFrequency:
		with: 999
		in: 'server'.
	{  'client1'. 'client2'  } do: 
		[ : each | 
		self 
			remotePerform: #startAddStrings
			in: each ].
	(Delay forMilliseconds: 3000) wait.	"kill before first flushCache, but some strings should be added within this time.."
	
	[ self 
		remotePerform: #killImage
		in: 'server' ] 
		on: MaNetworkError
		do: 
			[ : err | 
			"nothing, expected because we're killing the server."
			 ].
	self waitForServerDown.
	(Delay forSeconds: 5) wait.	"to give time for clients network timeout to expire."
	self class launchImageNamed: self class serverImageFilename.
	[ self class readyAt: 1316 ] whileFalse: 
		[ MagmaTesterNotification signal: 'waiting for server to recover..'.
		(Delay forSeconds: 2) wait ].
	"Our socket to communicate with the test-player hosting the server is now invalid, but the auto-retry logic should establish a new one automatically."
	self 
		remotePerform: #startServer
		in: 'server'.
	self 
		remotePerform: #assertForwardRecoverySignaled
		in: 'server'.
	self
		disconnectClients ;
		connectClients.
	{  'client1'. 'client2'  } do: 
		[ : each | 
		self 
			remotePerform: #verifyAddedStrings
			in: each ].
	InitiateBackupPrereqsCompleted add: #testAfterImagesRecovery! !

!MagmaTestCase methodsFor: 'test forwarding proxys' stamp: 'cmm 1/18/2005 12:34'!
testAutoConnectingForwardingProxy

	self
		createPeer ;
		remotePerform: #startPeer in: 'server' ;
		remotePerform: #commitForwardingProxy in: 'client1' ;
		remotePerform: #stopPeer in: 'server' ;
		remotePerform: #agitateRemoteProxy in: 'client2' ;
		remotePerform: #startPeer in: 'server' ;
		remotePerform: #agitateRemoteProxyAgain in: 'client2' ;
		remotePerform: #stopPeer in: 'server'! !

!MagmaTestCase methodsFor: 'test blocks' stamp: 'cmm 11/12/2004 16:46'!
testBlocks
	"We will simply use a two-arg sortBlock to test blocks."

	self
		remotePerform: #addSortBlock
		in: 'client1'.
	self
		remotePerform: #verifyMaterializedSortBlock
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 10/14/2004 12:51'!
testClassSchemaUpgrades
	"This method tests Magma's ability to run with a class-definition out-of-sync with the repository, as well as performing a live upgrade of MagmaPerson.

	- First, we commit initial MagmaPerson instances right here in the conductor image.
	- client1 adds an 'contactMethods' attribute in the image only as the first instVar of the superclass (MagmaContact).
	- client2 connects to the db, reads the MagmaPerson (without any contactMethods).
	- client1 sets the contactMethods, commits the latest code and the MagmaPerson.
	- client2 changes the name and commits, should get a commit-conflict, which causes the object to be refreshed (now known to be truncated).
	- client2 changes the name again and commits, should get a warning that data will be lost.  client2 aborts the commit.
	- client2 loads the latest code, since any commits are going to lose the address.
	- client2 stubsOut: the MagmaPerson to force a refresh of that object and clear the truncation flag.
	- client2 tries to change the name for a third time.  This time it should work.
	- client2 verifies the address added by client1.
	- client1 verifies the name change.
	- delete domain classes off the server, shutdown and reopen and ensure they don't get recompiled."

	self

		commitInitialMagmaPerson ;
		
		remotePerform: #addContactMethodsAttribute
		in: 'client1' ;

		remotePerform: #readPerson
		in: 'client2' ;
		
		remotePerform: #setContactMethodsAndCode
		in: 'client1' ;

		remotePerform: #changeName
		in: 'client2' ;

		remotePerform: #verifyNameChange
		in: 'client1'! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/21/2007 00:21'!
testCreateMagmaCollectionAndIndexSimultaneously
	self 
		remotePerform: #createMagmaCollectionAndIndexSimultaneously
		in: 'client1'.
	self 
		remotePerform: #verifyMagmaCollectionAndIndexCreatedSimultaneously
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test connectionless requests' stamp: 'cmm 7/15/2008 17:12'!
testDisconnectReconnect
	"Just disconnecting and reconnecting would force the magmaId to be validated."
	self 
		remotePerform: #performDisconnectReconnect
		in: 'client1'! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/21/2007 00:25'!
testDoubleUpdate
	"Make sure that when an object link is updated more than once before another client refreshes it, that it refreshes correctly."
	self 
		remotePerform: #performDoubleUpdate
		in: 'client1'.
	self 
		remotePerform: #verifyDoubleUpdate
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 5/8/2008 22:16'!
testGetServerStatistics
	"For now, just see if we can retrieve the console statistics"
	| session serverStats |
	session := self class magmaServerLocation newSession.
	serverStats := session serverStatistics.
	self assert: serverStats class = MagmaServerStatistics.
	self assert: (serverStats historys allSatisfy: [ : each | (each instVarNamed: 'timer') isNil ])! !

!MagmaTestCase methodsFor: 'test grow an object' stamp: 'cmm 7/16/2002 22:38'!
testGrowObject
	"Magma has special-case code on the server (see MaObjectFiler>>#write:index:) that checks whether an object now takes up more space than it used to.  This can occur when a variable is added to a class, or when a Collection had to grow its internal Array.  This test exercises that code."

	self
		remotePerform: #addAndGrowObject
		in: 'client1'.
	self
		remotePerform: #readAndVerifyGrownObject
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 1/18/2007 23:32'!
testLocalSessionsConcurrencyDetection
	self 
		remotePerform: #runLocalSessionsConcurrencyDetection
		in: 'server'! !

!MagmaTestCase methodsFor: 'test MagmaCounter' stamp: 'cmm 7/20/2007 11:59'!
testMagmaCounter
	"- make sure no commit-conflict.
- verify value"
	self
		remotePerform: #createCounter
			in: 'client1' ;
		remotePerform: #validateAndIncrementCounter
			in: 'client2' ;
		remotePerform: #validateAndDecrementCounter
			in: 'client1' ;
		remotePerform: #finalValidateCounter
			in: 'client2'! !

!MagmaTestCase methodsFor: 'test projects' stamp: 'cmm 2/9/2005 16:34'!
testProjects

	self
		remotePerform: #expungeProjects in: 'client2' ;
		remotePerform: #commitProjects in: 'client1' ;
		remotePerform: #enterProjects in: 'client2'! !

!MagmaTestCase methodsFor: 'test refreshAll' stamp: 'cmm 2/11/2008 22:19'!
testRefreshAll
	self 
		remotePerform: #createObjectToBeDormantInImage
		in: 'client1'.
	self 
		remotePerform: #cacheDormantAndDisconnect
		in: 'client2'.
	self 
		remotePerform: #updateDormantObject
		in: 'client1'.
	self 
		remotePerform: #reconnectAndVerifyDormantUpdated
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 6/22/2007 14:01'!
testRollbackRecovery
	"- shutdown server.  Wait for shutdown.
	- build a Dictionary of filenames and their contents-hash using SecureHashAlgorithm.
	- simulate an outage in the middle of a juicy commit which includes added and removed keys to an existing MagmaCollection, and MagmaArray, new objects, a new class-def, and a new MagmaCollection (so we test it remove files.
	- wait for server to stop responding
	- restart server image, recovery will happen AUTOMATICALLY!!
	- wait for server to respond.
	- shutdown server.
	- use same method to build another Dictionary of the filenames->contents-hashes
	- assert: the first Dictionary = the second."
	| initialHashes postHashes |
	self 
		remotePerform: #buildIndexedMagmaCollectionFrom:at:
		withArguments: { 
				Collection.
				'existing-collection for commitDisaster'
			 }
		in: 'client1'.
	self 
		remotePerform: #buildExistingObjectsForCommitDisaster
		in: 'client2'.
	self 
		remotePerform: #buildMagmaArray
		in: 'client1'.
	self disconnectClients.
	initialHashes _ self 
		remotePerform: #captureFileChecksumsWithCopy:
		with: true
		in: 'server'.
	self connectClients.
	self 
		remotePerform: #setSimulateServerOutage:
		withArguments: {  true  }
		in: 'server'.
	self 
		remotePerform: #commitDisaster
		in: 'client2'.
	self waitForServerDown.
	self class launchImageNamed: self class serverImageFilename.
	[ self class readyAt: 1316 ] whileFalse: 
		[ MagmaTesterNotification signal: 'waiting for server to recover..'.
		(Delay forSeconds: 2) wait ].
	self 
		remotePerform: #startServer
		in: 'server'.
	postHashes _ self 
		remotePerform: #captureFileChecksumsWithCopy:
		with: false
		in: 'server'.
	self assert: initialHashes = postHashes.
	"Since we killed the server, we know we must reconnect the clients."
	self
		disconnectClients ;
		connectClients.
	InitiateBackupPrereqsCompleted add: #testRollbackRecovery! !

!MagmaTestCase methodsFor: 'test objects' stamp: 'cmm 9/13/2004 00:21'!
testSamples

	self remotePerform: #addSamples in: 'client1'.
	self remotePerform: #verifySamples in: 'client2'! !

!MagmaTestCase methodsFor: 'test sequence-number generator' stamp: 'cmm 8/23/2007 13:08'!
testSequenceNumberGenerator
	self 
		remotePerform: #createSequenceNumberGenerator in: 'client2' ;
		remotePerform: #startGeneratingSequenceNumbers in: 'client1' ;
		remotePerform: #startGeneratingSequenceNumbers in: 'client2'.
	(Delay forSeconds: 10) wait.
	self
		remotePerform: #stopGeneratingSequenceNumbers in: 'client1' ;
		remotePerform: #stopGeneratingSequenceNumbers in: 'client2' ;
		verifySequenceNumbers! !

!MagmaTestCase methodsFor: 'test Set' stamp: 'cmm 11/25/2004 19:57'!
testSet
	"Set and Dictionary are special because their instances are serialized in their logical form.  For Dictionary, a MaVariableObjectBuffer with oids paired for key, value, key, value,... etc."
	self
		remotePerform: #createTestSet
		in: 'client1'.
	self
		remotePerform: #verifyTestSet
		in: 'client2'.! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/21/2007 00:29'!
testStaleProxyAutoReconnect
	"When a session disconnects, any existing proxies that are hit later will first look for an existing session connected to the same server, or, if not found there, try to reconnect the one it has."
	self 
		remotePerform: #performSetUpStaleProxyAutoReconnect
		in: 'client1'.
	self 
		remotePerform: #verifySetUpStaleProxyAutoReconnect
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test storing a session' stamp: 'cmm 1/15/2007 18:37'!
testStoringMagmaSession
	self
		remotePerform: #commitReferenceToSession
		in: 'client1'.
	self
		remotePerform: #verifyReferenceToSession
		in: 'client2'! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/5/2007 00:00'!
testTransactions
	"This tests Magma's transaction processing, its ability to detect commit conflicts, properly refresh each client image based on MagmaOptions>>refreshPersistentObjectsEvenWhenChangedOnlyByMe."
	self
		addPeople ;
		addLocations ;
		performCommitConflictTest ;
		performCommitAndRefreshTest ;
		performBuildIndexCommitConflictTest! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/22/2007 11:21'!
testUpdateByteObjects
	"Make sure that when an object link is updated more than once before another client refreshes it, that it refreshes correctly."
	"Client1, commit a couple of byte objects."
	self 
		remotePerform: #prepareUpdateByteObjects
		in: 'client1'.
	"Client2, commit an update the byte objects.."
	self 
		remotePerform: #performUpdateByteObjects
		in: 'client2'.
	"client1, verify the byte objects.."
	self 
		remotePerform: #verifyUpdateByteObjects
		in: 'client1'! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/4/2007 23:55'!
update34990
	clientSession commit: 
		[ ((clientSession root at: 'BuildIndexCommitConflictTest_objects') at: 6979) key: 34991 ]! !

!MagmaTestCase methodsFor: 'test refreshAll' stamp: 'cmm 2/11/2008 22:16'!
updateDormantObject
	clientSession commit: 
		[ (clientSession root at: #testRefreshAll) 
			at: 1
			put: 'the updated value' ]! !

!MagmaTestCase methodsFor: 'test MagmaCounter' stamp: 'cmm 7/21/2007 15:29'!
validateAndDecrementCounter
	| counter |
	clientSession begin.
	counter _ self counterFixture.
	self assert: counter value = 45.
	counter decrement: 15.
	self assert: counter value = 30.
	clientSession commit.
	self assert: counter value = 30! !

!MagmaTestCase methodsFor: 'test MagmaCounter' stamp: 'cmm 7/24/2007 21:07'!
validateAndIncrementCounter
	| counter |
	clientSession begin.
	counter _ self counterFixture.
	self assert: counter value = 10.
	counter increment: 35.
	self assert: counter value = 45.
	clientSession commitAndBegin! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/22/2007 12:18'!
verifyAbortRefreshes
	| people |
	people _ clientSession root at: 'people'.
	clientSession abort.
	self should: [ (people at: 1) name first = 'elizabeth' ]! !

!MagmaTestCase methodsFor: 'test after-images recovery' stamp: 'cmm 5/2/2007 22:13'!
verifyAddedStrings

	"We can't know whether the last commit actually succeeded and on its way back before the server was killed.  If it did, then the local will be one behind the remote.  Otherwise, they'll be equal."
	self assert: afterImagesProgress size > 3.
	self assert: ((self remoteStringCollection size - afterImagesProgress size) between: 0 and: 1)! !

!MagmaTestCase methodsFor: 'testBecome' stamp: 'cmm 12/7/2004 14:42'!
verifyBecome

	clientSession abort.
	self assert: (clientSession root at: #testBecomeToTransient) = { 'new'. 'becomed'. 'array' }.
	"remote becomes cannot be detected from transient objects."
	self assert: (localRoot at: #testBecomeToTransient) = { 'become to transient' }.

	self assert: (clientSession root at: #testBecomeToPersistent) = { 'becomee with different size'.  'becomee with different size' }! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/22/2007 12:17'!
verifyCommitAlsoRefreshes
	| people |
	people _ clientSession root at: 'people'.
	clientSession commit.
	self should: [ (people at: 2) spouse == (people at: 3) ].
	self should: [ (people at: 2) == (people at: 3) spouse ].
	self should: [ (people at: 3) name first = 'smitty' ].
	self should: [ (people at: 1) name first = 'elizabeth' ]! !

!MagmaTestCase methodsFor: 'test transactions' stamp: 'cmm 6/22/2007 12:23'!
verifyCommitFailure
	| paula chris |
	paula _ (clientSession root at: 'people') first.
	chris _ (clientSession root at: 'people') at: 2.
	self should: 
		[ 
		[ clientSession commit.
		false ] 
			on: MagmaCommitError
			do: 
				[ : exception | 
				self
					should: [ exception result commitConflicts size = 1 ] ;
					should: 
						[ exception result commitConflicts first conflictingConnection userId = 'client1' ] ;
					should: [ exception result commitConflicts first conflictingObjects size = 1 ] ;
					should: 
						[ exception result commitConflicts first conflictingObjects includes: paula ] ;
					should: [ chris spouse == paula ] ;
					should: [ paula spouse == chris ].
				true ] ]! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/21/2007 00:25'!
verifyDoubleUpdate
	clientSession abort.
	self should: [ (clientSession root at: 'testDoubleUpdate') = (5 -> 6) ]! !

!MagmaTestCase methodsFor: 'test commit object knot' stamp: 'cmm 9/13/2004 00:23'!
verifyKnot

	| sizes |
	self clientSession abort.  "refresh view"
	sizes _ self clientSession root at: 'knotty object sizes'.
	(self clientSession root at: 'a knotty object') withIndexDo:
		[ : e : index |
		e printString.
		self should: [ e size = (sizes at: index) size ] ].
	^nil! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/21/2007 00:21'!
verifyMagmaCollectionAndIndexCreatedSimultaneously
	clientSession abort.
	self should: 
		[ (clientSession root at: 'testCreateMagmaCollectionAndIndexSimultaneously') indexes notEmpty ]! !

!MagmaTestCase methodsFor: 'test blocks' stamp: 'cmm 11/12/2004 17:11'!
verifyMaterializedSortBlock

	| collection |
	collection _ MaObject withAllSubclasses.
	self should:
		[ "We have to convert each sortedCollection to Array's because BlockContexts do not support equality testing."
		(collection asSortedCollection: self sortBlock) asArray = (collection asSortedCollection: (clientSession root at: 'testBlocks')) asArray ]! !

!MagmaTestCase methodsFor: 'test class schema upgrades' stamp: 'cmm 6/20/2004 22:35'!
verifyNameChange

	clientSession abort.
	self should: [ (clientSession root at: 'testClassSchemaUpgrades') name = 'Muller' ]! !

!MagmaTestCase methodsFor: 'test storing a session' stamp: 'cmm 1/15/2007 18:39'!
verifyReferenceToSession
	clientSession abort.
	self assert: (clientSession root at: 'testStoringMagmaSession') == clientSession! !

!MagmaTestCase methodsFor: 'test backup/restore' stamp: 'cmm 7/1/2007 20:34'!
verifyRestoreIfTime
	timeToVerifyRestore ifTrue: [ self verifyRestore ]! !

!MagmaTestCase methodsFor: 'test objects' stamp: 'cmm 2/3/2005 23:59'!
verifySamples

	| newSamples materializedSamples |
	clientSession abort.
	newSamples _ MaFixtureFactory new samples.
	(materializedSamples _ clientSession root at: 'samples') keysAndValuesDo:
		[ : eachSelector : eachSamples |
		eachSamples with: (newSamples at: eachSelector) do:
			[ : eachMaterialized : eachNew |
			self should: [ eachMaterialized maEquivalentForSerializationTest: eachNew ] ] ].
	(newSamples at: #specialGlobalCollections)
		with: (materializedSamples at: #specialGlobalCollections)
		do:
			[ : eachInImage : eachMaterialized |
			self should: [ eachInImage == eachMaterialized ] ]! !

!MagmaTestCase methodsFor: 'test sequence-number generator' stamp: 'cmm 8/23/2007 16:03'!
verifySequenceNumbers
	| client1Numbers client2Numbers serverCounterValue |
	serverCounterValue _ self remotePerform: #getServerCounterValue in: 'client1'.
	client1Numbers _ self remotePerform: #getSequenceNumbers in: 'client1'.
	client2Numbers _ self remotePerform: #getSequenceNumbers in: 'client2'.
	self assert: (1 to: serverCounterValue by: 1) asArray = ((client1Numbers, client2Numbers) asSortedCollection asArray)! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 2/13/2008 21:00'!
verifySetUpStaleProxyAutoReconnect
	localRoot 
		at: 'testStaleProxyAutoReconnect'
		put: (clientSession abort root at: 'testStaleProxyAutoReconnect').
	self should: [ (localRoot at: 'testStaleProxyAutoReconnect') name maIsMutatingProxy ].
	clientSession disconnect.
	self should: 
		[ (localRoot at: 'testStaleProxyAutoReconnect') name asUppercase = 'BENNY' ]! !

!MagmaTestCase methodsFor: 'test Set' stamp: 'cmm 6/16/2005 11:49'!
verifyTestSet
	"Sets and Dictionary's are supposed to read one-level deep no matter what."
	| holder |
	holder _ (clientSession abort root at: 'testSet') anyOne.
	self 
		should: [ holder species = StringHolder ] ;
		should: [ holder contents maIsMutatingProxy ] ;
		should: [ holder contents = 'proxied string' ]! !

!MagmaTestCase methodsFor: 'other small tests' stamp: 'cmm 6/21/2007 00:36'!
verifyUpdateByteObjects
	clientSession abort.
	self should: [ (clientSession root at: 'testUpdateByteObjects') first first == $H ].
	self should: 
		[ (clientSession root at: 'testUpdateByteObjects') last first == $H asciiValue ]! !

!MagmaTestCase methodsFor: 'test rollback recovery' stamp: 'cmm 6/7/2005 10:28'!
waitForServerDown

	[ (Delay forSeconds: 2) wait.
	self class readyAt: 51969 ] 
		whileTrue:
			[ MagmaTesterNotification signal: 'Waiting for server to shutdown..' ]! !

!MagmaTestCase methodsFor: 'test become' stamp: 'cmm 1/15/2005 13:47'!
xtestBecome
self maMarked: 'dev'.  "This test fails.  Magma is not expecting differently shaped objects when it refreshes its objects.  For the future..  maybe."
	self
		remotePerform: #setUpTestBecome
		in: 'client1'.
	self
		remotePerform: #become
		in: 'client2'.
	self
		remotePerform: #verifyBecome
		in: 'client1'! !

!MagmaTestCase methodsFor: 'test commit object knot' stamp: 'cmm 9/13/2004 14:52'!
xtestCommitKnot
	"This method tests Magma's ability to commit a tangled knot of an
	object structure by one client, then retrieved and compared, as
	best as possible, against a dynamically created version of the
	same knot in client2 (see MaObjectGraphComparer for this)."

	self remotePerform: #addKnot in: 'client1'.
	self remotePerform: #verifyKnot in: 'client2'! !

MaObject subclass: #MagmaMailingAddress
	instanceVariableNames: 'street zip'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma tester-Test domains'!

!MagmaMailingAddress methodsFor: 'accessing' stamp: ''!
street

	^street! !

!MagmaMailingAddress methodsFor: 'building' stamp: ''!
street: aString

	street _ aString
! !

!MagmaMailingAddress methodsFor: 'accessing' stamp: ''!
zip

	^zip! !

!MagmaMailingAddress methodsFor: 'building' stamp: ''!
zip: aString

	zip _ aString
! !

MaObject subclass: #MagmaPersonName
	instanceVariableNames: 'first middle last'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma tester-Test domains'!

!MagmaPersonName methodsFor: 'accessing' stamp: ''!
first

	^first! !

!MagmaPersonName methodsFor: 'building' stamp: ''!
first: aString

	first _ aString
! !

!MagmaPersonName methodsFor: 'accessing' stamp: ''!
last

	^last! !

!MagmaPersonName methodsFor: 'building' stamp: ''!
last: aString

	last _ aString
! !

!MagmaPersonName methodsFor: 'printing' stamp: 'cmm 4/22/2002 22:25'!
maPrintAbbreviatedOn: aStream

	aStream
		nextPutAll: self first;
		space;
		nextPutAll: self last
! !

!MagmaPersonName methodsFor: 'accessing' stamp: ''!
middle

	^middle! !

!MagmaPersonName methodsFor: 'building' stamp: ''!
middle: aString

	middle _ aString
! !

MaObject subclass: #MagmaTestObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma tester-Test domains'!

MagmaTestObject subclass: #MagmaContact
	instanceVariableNames: 'name address unused'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma tester-Test domains'!

!MagmaContact methodsFor: 'accessing' stamp: 'cmm 6/7/2004 22:31'!
address

	^address! !

!MagmaContact methodsFor: 'as yet unclassified' stamp: 'cmm 6/7/2004 22:31'!
address: aMagmaMailingAddress

	address _ aMagmaMailingAddress! !

!MagmaContact methodsFor: 'accessing' stamp: 'cmm 6/7/2004 22:28'!
name
	^name! !

!MagmaContact methodsFor: 'accessing' stamp: 'cmm 6/7/2004 22:28'!
name: aString

	name _ aString! !

MagmaContact subclass: #MagmaPerson
	instanceVariableNames: 'birthday spouse'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma tester-Test domains'!

!MagmaPerson methodsFor: 'accessing' stamp: ''!
birthday

	^birthday! !

!MagmaPerson methodsFor: 'building' stamp: ''!
birthday: aMaTimeStamp

	birthday _ aMaTimeStamp
! !

!MagmaPerson methodsFor: 'printing' stamp: 'cmm 12/2/2004 16:43'!
maPrintAbbreviatedOn: aStream

	self name maPrintAbbreviatedOn: aStream
! !

!MagmaPerson methodsFor: 'actions' stamp: ''!
marry: aMagmaPerson

	self
		spouse: aMagmaPerson;
		moveInWith: aMagmaPerson
! !

!MagmaPerson methodsFor: 'actions' stamp: ''!
moveInWith: aMagmaPerson

	self address: aMagmaPerson address
! !

!MagmaPerson methodsFor: 'accessing' stamp: ''!
spouse

	^spouse! !

!MagmaPerson methodsFor: 'building' stamp: ''!
spouse: aMagmaPerson

	spouse _ aMagmaPerson.
	aMagmaPerson spouse == self ifFalse: [ aMagmaPerson spouse: self ]
! !

!MagmaTestObject methodsFor: 'as yet unclassified' stamp: 'cmm 10/2/2004 10:38'!
doesNotUnderstand: aMessage

	^ aMessage selector last = $:
		ifTrue:
			[ self
				instVarNamed: (aMessage selector copyWithout: $:)
				put: aMessage argument;
				yourself ]
		ifFalse:
			[ self instVarNamed: aMessage selector ]! !

!MagmaCollectionIndex methodsFor: '*magma tester' stamp: 'cmm 6/14/2006 12:49'!
ascendingSortBlock
	^ [ :a :b | (self indexHashFor: a) < (self indexHashFor: b) ]! !

!MagmaCollectionIndex methodsFor: '*magma tester' stamp: 'cmm 6/14/2006 12:49'!
descendingSortBlock
	^ [ :a :b | (self indexHashFor: a) > (self indexHashFor: b) ]! !

PackageInfo subclass: #MagmaTesterPackageInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma tester-Info'!

!MagmaTesterPackageInfo methodsFor: 'as yet unclassified' stamp: 'cmm 6/7/2004 23:26'!
additionalFiles

	^#('MagmaContact.1.st')! !

!MagmaTesterPackageInfo methodsFor: 'as yet unclassified' stamp: 'cmm 9/14/2004 00:23'!
changesText

	^'
1.0gamma8
	- Now has 1-click kickoff of the test cases by evaluating "MagmaTestCase kickoff".  Now requires OSProcess to accomplish this.
	- Tremendous increase in stringency of Morph test by storing and retrieving a MauiCollectionMorph instead of a single RectangleMorph.
	- Added tests involving MagmaFixtureFactory to generate all possible objects Magma supports (requires DictionaryEquivalencePerformanceBoost).

1.0gamma7
	- Extra stringent tests for MaHashIndex and MaHashIndexRecord ensure ultra-reliable support for MagmaCollections.'! !

!MagmaTesterPackageInfo methodsFor: 'accessing' stamp: 'cmm 11/2/2004 16:43'!
prerequisitePackages
	^ #( 'Ma Armored Code' 'Magma server' 'Ma Squeak domain' 'Maui' 'Ma object serialization tester' )! !

!MagmaTesterPackageInfo methodsFor: 'as yet unclassified' stamp: 'cmm 10/7/2004 22:13'!
readmeText

	^'This package runs rigorous SUnit test scripts against Magma.

Temporary repository files must be created to run the test.  They will be created in a directory called ''Magma'' in the default directory unless you change it by executing:

	MaArmoredCodeOptions current testEnvironmentPath: (FileDirectory on: ''c:\temp\'')

When running the test cases, it is recommended to turn on debug mode.  Then, if an exception occurs in the server, it will halt there instead of returning that exception to the client and signaling it there.

	MagmaPreferences debug: true; debugProxies: false

  To kick off the test suite, execute:
	
	MagmaTestCase kickoff
'! !

!MaObjectRepository class methodsFor: '*magma tester' stamp: 'cmm 7/13/2005 17:31'!
runningTestCases: aBoolean

	RunningTestCases _ aBoolean! !

!MaObjectRepository class methodsFor: '*magma tester' stamp: 'cmm 4/29/2007 15:06'!
simulateOutage
	^ SimulateOutage! !

!MaObjectRepository class methodsFor: '*magma tester' stamp: 'cmm 6/1/2005 16:13'!
simulateOutage: aBoolean
	"this will cause the server to exit the Squeak image in the middle of a commit so that the fault-tolerance feature can be tested."

	SimulateOutage _ aBoolean! !

!MaObjectRepository methodsFor: '*magma tester' stamp: 'cmm 6/17/2005 14:01'!
simulateOutage

	MagmaServerPreferences debugRecovery ifTrue: [ self halt: 'about to kill image' ]. 
	SmalltalkImage current quitPrimitive! !

!ClassDescription methodsFor: '*magma tester' stamp: 'cmm 12/16/2002 14:34'!
maCommentKeywords

	^self comment asString substrings! !

!MagmaCollectionReader methodsFor: '*magma tester' stamp: 'cmm 7/4/2006 22:46'!
sortBlock
	^ self isDescending 
		ifTrue: [ self sortIndex descendingSortBlock ]
		ifFalse: [ self sortIndex ascendingSortBlock ]! !

MaTestCase subclass: #MagmaUtilitiesTester
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma tester-SUnit tests'!

!MagmaUtilitiesTester methodsFor: 'private' stamp: 'cmm 9/26/2004 19:38'!
compressedRepositoryPath

	^ (FileDirectory on: super environmentPath) fullNameFor: 'compressed'! !

!MagmaUtilitiesTester methodsFor: 'private' stamp: 'cmm 10/7/2004 17:01'!
environmentPath

	^ self environmentDirectory fullNameFor: 'Magma'! !

!MagmaUtilitiesTester methodsFor: 'private' stamp: 'cmm 10/7/2004 17:03'!
loadChangeSet: fileName

	| oldChanges file |
	oldChanges _ ChangeSet current.
	ChangeSet newChanges: (ChangeSorter assuredChangeSetNamed: 'MagmaTestClasses').
	file _ self resourcesDirectory readOnlyFileNamed: fileName.
	[ file 
		maFileInAnnouncing: 'Loading ', file localName
		logged: false ] 

		ensure:
			[ file close.
			ChangeSet newChanges: oldChanges ]! !

!MagmaUtilitiesTester methodsFor: 'private' stamp: 'cmm 10/7/2004 17:28'!
projectDirectory

	^ (self projectsDirectory directoryExists: 'Magma')
		ifTrue:
			[ FileDirectory on: (self projectsDirectory fullNameFor: 'Magma') ]
		ifFalse:
			[ self projectsDirectory ]! !

!MagmaUtilitiesTester methodsFor: 'tests' stamp: 'cmm 7/16/2007 22:10'!
testCompress
	| session garbageOid goodOid |
	session _ MagmaSession openLocal: self environmentPath.
	session connect: (MagmaUser id: 'testCompress').
	session commit: 
		[ session root 
			at: 'testCompress'
			put: (1 to: 100) asArray ].
	garbageOid _ session oidFor: (session root at: 'testCompress').
	session commit: 
		[ session root 
			at: 'testCompress'
			put: (100 to: 200) asArray ].
	goodOid _ session oidFor: (session root at: 'testCompress').
	self should: [ garbageOid ~= goodOid ].
	session disconnectAndClose.
	[ MagmaRepositoryController
		delete: self compressedRepositoryPath ;
		compress: self environmentPath
			to: self compressedRepositoryPath ] 
		on: MagmaDeleteWarning
		do: [ : warning | warning resume ].
	session _ MagmaSession openLocal: self compressedRepositoryPath.
	session connect: (MagmaUser id: 'chris').
	self should: [ (session root at: 'testCompress') = (100 to: 200) asArray ].
	self should: [ (session 
			objectWithOid: garbageOid
			ifAbsent: [ nil ]) isNil ].
	self should: [ (session 
			objectWithOid: goodOid
			ifAbsent: [ nil ]) notNil ].
	session closeRepository! !

!MagmaUtilitiesTester methodsFor: 'tests' stamp: 'cmm 9/21/2005 13:53'!
testLargeCollectionOfIntegers

	| lci filename |
	filename := MaArmoredCodeOptions current testEnvironmentDirectory fullNameFor: 'lcitest.idx'.
	"ensure deleted.."
	MaArmoredCodeOptions current testEnvironmentDirectory deleteFileNamed: 'lcitest.idx' ifAbsent: [ "no problemo" ].
	lci := MaLargeArrayOfNumbers createNamed: filename bitSize: MaObjectFiler maxDbSize.
	lci
		at: 10000
		put: 33.
	1 to: 9999 do: [ : n | self assert: (lci at: n) = 0 ]! !
MagmaCollectionTester initialize!
